import inspect
from typing import Union, List, Iterable, Callable

from temporal.api.common.v1 import Payload, Payloads
from temporal.conversions import ENCODINGS, METADATA_ENCODING_KEY, DECODINGS


def get_fn_args_type_hints(fn: Callable) -> List[type]:
    spec = inspect.getfullargspec(fn)
    args = []
    for s in spec.args:
        args.append(spec.annotations.get(s))
    if inspect.ismethod(fn):
        # Remove "self
        args.pop(0)
    return args


def get_fn_ret_type_hints(fn: Callable) -> List[type]:
    spec = inspect.getfullargspec(fn)
    return [spec.annotations.get("return")]


class DataConverter:
    def to_payload(self, arg: object) -> Payload:
        raise NotImplementedError()

    def from_payload(self, payload: Payload, type_hint: type = None) -> object:
        raise NotImplementedError()

    def to_payloads(self, args: Union[object, List[object]]) -> Payloads:
        payloads: Payloads = Payloads()
        payloads.payloads = []
        if isinstance(args, (str, bytes)) or not isinstance(args, Iterable):
            args = [args]
        for arg in args:
            payloads.payloads.append(self.to_payload(arg))
        return payloads

    def from_payloads(self, payloads: Payloads, type_hints: List[type] = []) -> List[object]:
        args: List[object] = []
        for i, payload in enumerate(payloads.payloads):
            try:
                type_hint = type_hints[i]
            except IndexError as ex:
                type_hint = None
            args.append(self.from_payload(payload, type_hint))
        return args

    @staticmethod
    def get_default():
        return DefaultDataConverter()


class DefaultDataConverter(DataConverter):
    def to_payload(self, arg: object) -> Payload:
        for fn in ENCODINGS:
            payload = fn(arg)
            if payload is not None:
                return payload
        raise Exception(f"Object cannot be encoded: {arg}")

    def from_payload(self, payload: Payload, type_hint: type = None) -> object:
        encoding: bytes = payload.metadata[METADATA_ENCODING_KEY]
        decoding = DECODINGS.get(encoding)
        if not decoding:
            raise Exception(f"Unsupported encoding: {str(encoding, 'utf-8')}")
        return decoding(payload)


DEFAULT_DATA_CONVERTER_INSTANCE = DefaultDataConverter()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: temporal/api/history/v1/message.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime, timedelta
from typing import Dict, List

import betterproto

from temporal.api.common import v1 as v1common
from temporal.api.enums import v1 as v1enums
from temporal.api.failure import v1 as v1failure
from temporal.api.taskqueue import v1 as v1taskqueue
from temporal.api.workflow import v1 as v1workflow


@dataclass
class WorkflowExecutionStartedEventAttributes(betterproto.Message):
    workflow_type: v1common.WorkflowType = betterproto.message_field(1)
    parent_workflow_namespace: str = betterproto.string_field(2)
    parent_workflow_execution: v1common.WorkflowExecution = betterproto.message_field(3)
    parent_initiated_event_id: int = betterproto.int64_field(4)
    task_queue: v1taskqueue.TaskQueue = betterproto.message_field(5)
    input: v1common.Payloads = betterproto.message_field(6)
    # Total workflow execution timeout including retries and continue as new.
    workflow_execution_timeout: timedelta = betterproto.message_field(7)
    # Timeout of a single workflow run.
    workflow_run_timeout: timedelta = betterproto.message_field(8)
    # Timeout of a single workflow task.
    workflow_task_timeout: timedelta = betterproto.message_field(9)
    continued_execution_run_id: str = betterproto.string_field(10)
    initiator: v1enums.ContinueAsNewInitiator = betterproto.enum_field(11)
    continued_failure: v1failure.Failure = betterproto.message_field(12)
    last_completion_result: v1common.Payloads = betterproto.message_field(13)
    # This is the runId when the WorkflowExecutionStarted event is written.
    original_execution_run_id: str = betterproto.string_field(14)
    identity: str = betterproto.string_field(15)
    # This is the very first runId along the chain of ContinueAsNew and Reset.
    first_execution_run_id: str = betterproto.string_field(16)
    retry_policy: v1common.RetryPolicy = betterproto.message_field(17)
    attempt: int = betterproto.int32_field(18)
    # The absolute time at which workflow is timed out. This time is passed
    # without change to the next run/retry of a workflow.
    workflow_execution_expiration_time: datetime = betterproto.message_field(19)
    cron_schedule: str = betterproto.string_field(20)
    first_workflow_task_backoff: timedelta = betterproto.message_field(21)
    memo: v1common.Memo = betterproto.message_field(22)
    search_attributes: v1common.SearchAttributes = betterproto.message_field(23)
    prev_auto_reset_points: v1workflow.ResetPoints = betterproto.message_field(24)
    header: v1common.Header = betterproto.message_field(25)


@dataclass
class WorkflowExecutionCompletedEventAttributes(betterproto.Message):
    result: v1common.Payloads = betterproto.message_field(1)
    workflow_task_completed_event_id: int = betterproto.int64_field(2)


@dataclass
class WorkflowExecutionFailedEventAttributes(betterproto.Message):
    failure: v1failure.Failure = betterproto.message_field(1)
    retry_state: v1enums.RetryState = betterproto.enum_field(2)
    workflow_task_completed_event_id: int = betterproto.int64_field(3)


@dataclass
class WorkflowExecutionTimedOutEventAttributes(betterproto.Message):
    retry_state: v1enums.RetryState = betterproto.enum_field(1)


@dataclass
class WorkflowExecutionContinuedAsNewEventAttributes(betterproto.Message):
    new_execution_run_id: str = betterproto.string_field(1)
    workflow_type: v1common.WorkflowType = betterproto.message_field(2)
    task_queue: v1taskqueue.TaskQueue = betterproto.message_field(3)
    input: v1common.Payloads = betterproto.message_field(4)
    # workflow_execution_timeout is omitted as it shouldn'be overridden from
    # within a workflow. Timeout of a single workflow run.
    workflow_run_timeout: timedelta = betterproto.message_field(5)
    # Timeout of a single workflow task.
    workflow_task_timeout: timedelta = betterproto.message_field(6)
    workflow_task_completed_event_id: int = betterproto.int64_field(7)
    backoff_start_interval: timedelta = betterproto.message_field(8)
    initiator: v1enums.ContinueAsNewInitiator = betterproto.enum_field(9)
    failure: v1failure.Failure = betterproto.message_field(10)
    last_completion_result: v1common.Payloads = betterproto.message_field(11)
    header: v1common.Header = betterproto.message_field(12)
    memo: v1common.Memo = betterproto.message_field(13)
    search_attributes: v1common.SearchAttributes = betterproto.message_field(14)


@dataclass
class WorkflowTaskScheduledEventAttributes(betterproto.Message):
    task_queue: v1taskqueue.TaskQueue = betterproto.message_field(1)
    # (-- api-linter: core::0140::prepositions=disabled     aip.dev/not-
    # precedent: "to" is used to indicate interval. --)
    start_to_close_timeout: timedelta = betterproto.message_field(2)
    attempt: int = betterproto.int32_field(3)


@dataclass
class WorkflowTaskStartedEventAttributes(betterproto.Message):
    scheduled_event_id: int = betterproto.int64_field(1)
    identity: str = betterproto.string_field(2)
    request_id: str = betterproto.string_field(3)


@dataclass
class WorkflowTaskCompletedEventAttributes(betterproto.Message):
    scheduled_event_id: int = betterproto.int64_field(1)
    started_event_id: int = betterproto.int64_field(2)
    identity: str = betterproto.string_field(3)
    binary_checksum: str = betterproto.string_field(4)


@dataclass
class WorkflowTaskTimedOutEventAttributes(betterproto.Message):
    scheduled_event_id: int = betterproto.int64_field(1)
    started_event_id: int = betterproto.int64_field(2)
    timeout_type: v1enums.TimeoutType = betterproto.enum_field(3)


@dataclass
class WorkflowTaskFailedEventAttributes(betterproto.Message):
    scheduled_event_id: int = betterproto.int64_field(1)
    started_event_id: int = betterproto.int64_field(2)
    cause: v1enums.WorkflowTaskFailedCause = betterproto.enum_field(3)
    failure: v1failure.Failure = betterproto.message_field(4)
    identity: str = betterproto.string_field(5)
    # For reset workflow.
    base_run_id: str = betterproto.string_field(6)
    new_run_id: str = betterproto.string_field(7)
    fork_event_version: int = betterproto.int64_field(8)
    binary_checksum: str = betterproto.string_field(9)


@dataclass
class ActivityTaskScheduledEventAttributes(betterproto.Message):
    activity_id: str = betterproto.string_field(1)
    activity_type: v1common.ActivityType = betterproto.message_field(2)
    namespace: str = betterproto.string_field(3)
    task_queue: v1taskqueue.TaskQueue = betterproto.message_field(4)
    header: v1common.Header = betterproto.message_field(5)
    input: v1common.Payloads = betterproto.message_field(6)
    # (-- api-linter: core::0140::prepositions=disabled     aip.dev/not-
    # precedent: "to" is used to indicate interval. --) Indicates how long the
    # caller is willing to wait for an activity completion. Limits for how long
    # retries are happening. Either this or start_to_close_timeout_seconds must
    # be specified.
    schedule_to_close_timeout: timedelta = betterproto.message_field(7)
    # (-- api-linter: core::0140::prepositions=disabled     aip.dev/not-
    # precedent: "to" is used to indicate interval. --) Limits time an activity
    # task can stay in a task queue before a worker picks it up. This timeout is
    # always non retryable as all a retry would achieve is to put it back into
    # the same queue. Defaults to schedule_to_close_timeout_seconds or workflow
    # execution timeout if not specified.
    schedule_to_start_timeout: timedelta = betterproto.message_field(8)
    # (-- api-linter: core::0140::prepositions=disabled     aip.dev/not-
    # precedent: "to" is used to indicate interval. --) Maximum time an activity
    # is allowed to execute after a pick up by a worker. This timeout is always
    # retryable. Either this or schedule_to_close_timeout_seconds must be
    # specified.
    start_to_close_timeout: timedelta = betterproto.message_field(9)
    # Maximum time between successful worker heartbeats.
    heartbeat_timeout: timedelta = betterproto.message_field(10)
    workflow_task_completed_event_id: int = betterproto.int64_field(11)
    # Activities are provided by a default retry policy controlled through the
    # service dynamic configuration. Retries are happening up to
    # schedule_to_close_timeout. To disable retries set
    # retry_policy.maximum_attempts to 1.
    retry_policy: v1common.RetryPolicy = betterproto.message_field(12)


@dataclass
class ActivityTaskStartedEventAttributes(betterproto.Message):
    scheduled_event_id: int = betterproto.int64_field(1)
    identity: str = betterproto.string_field(2)
    request_id: str = betterproto.string_field(3)
    attempt: int = betterproto.int32_field(4)
    last_failure: v1failure.Failure = betterproto.message_field(5)


@dataclass
class ActivityTaskCompletedEventAttributes(betterproto.Message):
    result: v1common.Payloads = betterproto.message_field(1)
    scheduled_event_id: int = betterproto.int64_field(2)
    started_event_id: int = betterproto.int64_field(3)
    identity: str = betterproto.string_field(4)


@dataclass
class ActivityTaskFailedEventAttributes(betterproto.Message):
    failure: v1failure.Failure = betterproto.message_field(1)
    scheduled_event_id: int = betterproto.int64_field(2)
    started_event_id: int = betterproto.int64_field(3)
    identity: str = betterproto.string_field(4)
    retry_state: v1enums.RetryState = betterproto.enum_field(5)


@dataclass
class ActivityTaskTimedOutEventAttributes(betterproto.Message):
    # For retry activity, it may have a failure before timeout. It is stored as
    # `cause` in `failure`.
    failure: v1failure.Failure = betterproto.message_field(1)
    scheduled_event_id: int = betterproto.int64_field(2)
    started_event_id: int = betterproto.int64_field(3)
    retry_state: v1enums.RetryState = betterproto.enum_field(4)


@dataclass
class ActivityTaskCancelRequestedEventAttributes(betterproto.Message):
    scheduled_event_id: int = betterproto.int64_field(1)
    workflow_task_completed_event_id: int = betterproto.int64_field(2)


@dataclass
class ActivityTaskCanceledEventAttributes(betterproto.Message):
    details: v1common.Payloads = betterproto.message_field(1)
    latest_cancel_requested_event_id: int = betterproto.int64_field(2)
    scheduled_event_id: int = betterproto.int64_field(3)
    started_event_id: int = betterproto.int64_field(4)
    identity: str = betterproto.string_field(5)


@dataclass
class TimerStartedEventAttributes(betterproto.Message):
    timer_id: str = betterproto.string_field(1)
    # (-- api-linter: core::0140::prepositions=disabled     aip.dev/not-
    # precedent: "to" is used to indicate interval. --)
    start_to_fire_timeout: timedelta = betterproto.message_field(2)
    workflow_task_completed_event_id: int = betterproto.int64_field(3)


@dataclass
class TimerFiredEventAttributes(betterproto.Message):
    timer_id: str = betterproto.string_field(1)
    started_event_id: int = betterproto.int64_field(2)


@dataclass
class TimerCanceledEventAttributes(betterproto.Message):
    timer_id: str = betterproto.string_field(1)
    started_event_id: int = betterproto.int64_field(2)
    workflow_task_completed_event_id: int = betterproto.int64_field(3)
    identity: str = betterproto.string_field(4)


@dataclass
class WorkflowExecutionCancelRequestedEventAttributes(betterproto.Message):
    cause: str = betterproto.string_field(1)
    external_initiated_event_id: int = betterproto.int64_field(2)
    external_workflow_execution: v1common.WorkflowExecution = betterproto.message_field(
        3
    )
    identity: str = betterproto.string_field(4)


@dataclass
class WorkflowExecutionCanceledEventAttributes(betterproto.Message):
    workflow_task_completed_event_id: int = betterproto.int64_field(1)
    details: v1common.Payloads = betterproto.message_field(2)


@dataclass
class MarkerRecordedEventAttributes(betterproto.Message):
    marker_name: str = betterproto.string_field(1)
    details: Dict[str, v1common.Payloads] = betterproto.map_field(
        2, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    workflow_task_completed_event_id: int = betterproto.int64_field(3)
    header: v1common.Header = betterproto.message_field(4)
    failure: v1failure.Failure = betterproto.message_field(5)


@dataclass
class WorkflowExecutionSignaledEventAttributes(betterproto.Message):
    signal_name: str = betterproto.string_field(1)
    input: v1common.Payloads = betterproto.message_field(2)
    identity: str = betterproto.string_field(3)


@dataclass
class WorkflowExecutionTerminatedEventAttributes(betterproto.Message):
    reason: str = betterproto.string_field(1)
    details: v1common.Payloads = betterproto.message_field(2)
    identity: str = betterproto.string_field(3)


@dataclass
class RequestCancelExternalWorkflowExecutionInitiatedEventAttributes(
    betterproto.Message
):
    workflow_task_completed_event_id: int = betterproto.int64_field(1)
    namespace: str = betterproto.string_field(2)
    workflow_execution: v1common.WorkflowExecution = betterproto.message_field(3)
    control: str = betterproto.string_field(4)
    child_workflow_only: bool = betterproto.bool_field(5)


@dataclass
class RequestCancelExternalWorkflowExecutionFailedEventAttributes(betterproto.Message):
    cause: v1enums.CancelExternalWorkflowExecutionFailedCause = betterproto.enum_field(
        1
    )
    workflow_task_completed_event_id: int = betterproto.int64_field(2)
    namespace: str = betterproto.string_field(3)
    workflow_execution: v1common.WorkflowExecution = betterproto.message_field(4)
    initiated_event_id: int = betterproto.int64_field(5)
    control: str = betterproto.string_field(6)


@dataclass
class ExternalWorkflowExecutionCancelRequestedEventAttributes(betterproto.Message):
    initiated_event_id: int = betterproto.int64_field(1)
    namespace: str = betterproto.string_field(2)
    workflow_execution: v1common.WorkflowExecution = betterproto.message_field(3)


@dataclass
class SignalExternalWorkflowExecutionInitiatedEventAttributes(betterproto.Message):
    workflow_task_completed_event_id: int = betterproto.int64_field(1)
    namespace: str = betterproto.string_field(2)
    workflow_execution: v1common.WorkflowExecution = betterproto.message_field(3)
    signal_name: str = betterproto.string_field(4)
    input: v1common.Payloads = betterproto.message_field(5)
    control: str = betterproto.string_field(6)
    child_workflow_only: bool = betterproto.bool_field(7)


@dataclass
class SignalExternalWorkflowExecutionFailedEventAttributes(betterproto.Message):
    cause: v1enums.SignalExternalWorkflowExecutionFailedCause = betterproto.enum_field(
        1
    )
    workflow_task_completed_event_id: int = betterproto.int64_field(2)
    namespace: str = betterproto.string_field(3)
    workflow_execution: v1common.WorkflowExecution = betterproto.message_field(4)
    initiated_event_id: int = betterproto.int64_field(5)
    control: str = betterproto.string_field(6)


@dataclass
class ExternalWorkflowExecutionSignaledEventAttributes(betterproto.Message):
    initiated_event_id: int = betterproto.int64_field(1)
    namespace: str = betterproto.string_field(2)
    workflow_execution: v1common.WorkflowExecution = betterproto.message_field(3)
    control: str = betterproto.string_field(4)


@dataclass
class UpsertWorkflowSearchAttributesEventAttributes(betterproto.Message):
    workflow_task_completed_event_id: int = betterproto.int64_field(1)
    search_attributes: v1common.SearchAttributes = betterproto.message_field(2)


@dataclass
class StartChildWorkflowExecutionInitiatedEventAttributes(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    workflow_id: str = betterproto.string_field(2)
    workflow_type: v1common.WorkflowType = betterproto.message_field(3)
    task_queue: v1taskqueue.TaskQueue = betterproto.message_field(4)
    input: v1common.Payloads = betterproto.message_field(5)
    # Total workflow execution timeout including retries and continue as new.
    workflow_execution_timeout: timedelta = betterproto.message_field(6)
    # Timeout of a single workflow run.
    workflow_run_timeout: timedelta = betterproto.message_field(7)
    # Timeout of a single workflow task.
    workflow_task_timeout: timedelta = betterproto.message_field(8)
    # Default: PARENT_CLOSE_POLICY_TERMINATE.
    parent_close_policy: v1enums.ParentClosePolicy = betterproto.enum_field(9)
    control: str = betterproto.string_field(10)
    workflow_task_completed_event_id: int = betterproto.int64_field(11)
    # Default: WORKFLOW_ID_REUSE_POLICY_ALLOW_DUPLICATE.
    workflow_id_reuse_policy: v1enums.WorkflowIdReusePolicy = betterproto.enum_field(12)
    retry_policy: v1common.RetryPolicy = betterproto.message_field(13)
    cron_schedule: str = betterproto.string_field(14)
    header: v1common.Header = betterproto.message_field(15)
    memo: v1common.Memo = betterproto.message_field(16)
    search_attributes: v1common.SearchAttributes = betterproto.message_field(17)


@dataclass
class StartChildWorkflowExecutionFailedEventAttributes(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    workflow_id: str = betterproto.string_field(2)
    workflow_type: v1common.WorkflowType = betterproto.message_field(3)
    cause: v1enums.StartChildWorkflowExecutionFailedCause = betterproto.enum_field(4)
    control: str = betterproto.string_field(5)
    initiated_event_id: int = betterproto.int64_field(6)
    workflow_task_completed_event_id: int = betterproto.int64_field(7)


@dataclass
class ChildWorkflowExecutionStartedEventAttributes(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    initiated_event_id: int = betterproto.int64_field(2)
    workflow_execution: v1common.WorkflowExecution = betterproto.message_field(3)
    workflow_type: v1common.WorkflowType = betterproto.message_field(4)
    header: v1common.Header = betterproto.message_field(5)


@dataclass
class ChildWorkflowExecutionCompletedEventAttributes(betterproto.Message):
    result: v1common.Payloads = betterproto.message_field(1)
    namespace: str = betterproto.string_field(2)
    workflow_execution: v1common.WorkflowExecution = betterproto.message_field(3)
    workflow_type: v1common.WorkflowType = betterproto.message_field(4)
    initiated_event_id: int = betterproto.int64_field(5)
    started_event_id: int = betterproto.int64_field(6)


@dataclass
class ChildWorkflowExecutionFailedEventAttributes(betterproto.Message):
    failure: v1failure.Failure = betterproto.message_field(1)
    namespace: str = betterproto.string_field(2)
    workflow_execution: v1common.WorkflowExecution = betterproto.message_field(3)
    workflow_type: v1common.WorkflowType = betterproto.message_field(4)
    initiated_event_id: int = betterproto.int64_field(5)
    started_event_id: int = betterproto.int64_field(6)
    retry_state: v1enums.RetryState = betterproto.enum_field(7)


@dataclass
class ChildWorkflowExecutionCanceledEventAttributes(betterproto.Message):
    details: v1common.Payloads = betterproto.message_field(1)
    namespace: str = betterproto.string_field(2)
    workflow_execution: v1common.WorkflowExecution = betterproto.message_field(3)
    workflow_type: v1common.WorkflowType = betterproto.message_field(4)
    initiated_event_id: int = betterproto.int64_field(5)
    started_event_id: int = betterproto.int64_field(6)


@dataclass
class ChildWorkflowExecutionTimedOutEventAttributes(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    workflow_execution: v1common.WorkflowExecution = betterproto.message_field(2)
    workflow_type: v1common.WorkflowType = betterproto.message_field(3)
    initiated_event_id: int = betterproto.int64_field(4)
    started_event_id: int = betterproto.int64_field(5)
    retry_state: v1enums.RetryState = betterproto.enum_field(6)


@dataclass
class ChildWorkflowExecutionTerminatedEventAttributes(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    workflow_execution: v1common.WorkflowExecution = betterproto.message_field(2)
    workflow_type: v1common.WorkflowType = betterproto.message_field(3)
    initiated_event_id: int = betterproto.int64_field(4)
    started_event_id: int = betterproto.int64_field(5)


@dataclass
class HistoryEvent(betterproto.Message):
    event_id: int = betterproto.int64_field(1)
    event_time: datetime = betterproto.message_field(2)
    event_type: v1enums.EventType = betterproto.enum_field(3)
    version: int = betterproto.int64_field(4)
    task_id: int = betterproto.int64_field(5)
    workflow_execution_started_event_attributes: "WorkflowExecutionStartedEventAttributes" = betterproto.message_field(
        6, group="attributes"
    )
    workflow_execution_completed_event_attributes: "WorkflowExecutionCompletedEventAttributes" = betterproto.message_field(
        7, group="attributes"
    )
    workflow_execution_failed_event_attributes: "WorkflowExecutionFailedEventAttributes" = betterproto.message_field(
        8, group="attributes"
    )
    workflow_execution_timed_out_event_attributes: "WorkflowExecutionTimedOutEventAttributes" = betterproto.message_field(
        9, group="attributes"
    )
    workflow_task_scheduled_event_attributes: "WorkflowTaskScheduledEventAttributes" = betterproto.message_field(
        10, group="attributes"
    )
    workflow_task_started_event_attributes: "WorkflowTaskStartedEventAttributes" = betterproto.message_field(
        11, group="attributes"
    )
    workflow_task_completed_event_attributes: "WorkflowTaskCompletedEventAttributes" = betterproto.message_field(
        12, group="attributes"
    )
    workflow_task_timed_out_event_attributes: "WorkflowTaskTimedOutEventAttributes" = betterproto.message_field(
        13, group="attributes"
    )
    workflow_task_failed_event_attributes: "WorkflowTaskFailedEventAttributes" = betterproto.message_field(
        14, group="attributes"
    )
    activity_task_scheduled_event_attributes: "ActivityTaskScheduledEventAttributes" = betterproto.message_field(
        15, group="attributes"
    )
    activity_task_started_event_attributes: "ActivityTaskStartedEventAttributes" = betterproto.message_field(
        16, group="attributes"
    )
    activity_task_completed_event_attributes: "ActivityTaskCompletedEventAttributes" = betterproto.message_field(
        17, group="attributes"
    )
    activity_task_failed_event_attributes: "ActivityTaskFailedEventAttributes" = betterproto.message_field(
        18, group="attributes"
    )
    activity_task_timed_out_event_attributes: "ActivityTaskTimedOutEventAttributes" = betterproto.message_field(
        19, group="attributes"
    )
    timer_started_event_attributes: "TimerStartedEventAttributes" = betterproto.message_field(
        20, group="attributes"
    )
    timer_fired_event_attributes: "TimerFiredEventAttributes" = betterproto.message_field(
        21, group="attributes"
    )
    activity_task_cancel_requested_event_attributes: "ActivityTaskCancelRequestedEventAttributes" = betterproto.message_field(
        22, group="attributes"
    )
    activity_task_canceled_event_attributes: "ActivityTaskCanceledEventAttributes" = betterproto.message_field(
        23, group="attributes"
    )
    timer_canceled_event_attributes: "TimerCanceledEventAttributes" = betterproto.message_field(
        24, group="attributes"
    )
    marker_recorded_event_attributes: "MarkerRecordedEventAttributes" = betterproto.message_field(
        25, group="attributes"
    )
    workflow_execution_signaled_event_attributes: "WorkflowExecutionSignaledEventAttributes" = betterproto.message_field(
        26, group="attributes"
    )
    workflow_execution_terminated_event_attributes: "WorkflowExecutionTerminatedEventAttributes" = betterproto.message_field(
        27, group="attributes"
    )
    workflow_execution_cancel_requested_event_attributes: "WorkflowExecutionCancelRequestedEventAttributes" = betterproto.message_field(
        28, group="attributes"
    )
    workflow_execution_canceled_event_attributes: "WorkflowExecutionCanceledEventAttributes" = betterproto.message_field(
        29, group="attributes"
    )
    request_cancel_external_workflow_execution_initiated_event_attributes: "RequestCancelExternalWorkflowExecutionInitiatedEventAttributes" = betterproto.message_field(
        30, group="attributes"
    )
    request_cancel_external_workflow_execution_failed_event_attributes: "RequestCancelExternalWorkflowExecutionFailedEventAttributes" = betterproto.message_field(
        31, group="attributes"
    )
    external_workflow_execution_cancel_requested_event_attributes: "ExternalWorkflowExecutionCancelRequestedEventAttributes" = betterproto.message_field(
        32, group="attributes"
    )
    workflow_execution_continued_as_new_event_attributes: "WorkflowExecutionContinuedAsNewEventAttributes" = betterproto.message_field(
        33, group="attributes"
    )
    start_child_workflow_execution_initiated_event_attributes: "StartChildWorkflowExecutionInitiatedEventAttributes" = betterproto.message_field(
        34, group="attributes"
    )
    start_child_workflow_execution_failed_event_attributes: "StartChildWorkflowExecutionFailedEventAttributes" = betterproto.message_field(
        35, group="attributes"
    )
    child_workflow_execution_started_event_attributes: "ChildWorkflowExecutionStartedEventAttributes" = betterproto.message_field(
        36, group="attributes"
    )
    child_workflow_execution_completed_event_attributes: "ChildWorkflowExecutionCompletedEventAttributes" = betterproto.message_field(
        37, group="attributes"
    )
    child_workflow_execution_failed_event_attributes: "ChildWorkflowExecutionFailedEventAttributes" = betterproto.message_field(
        38, group="attributes"
    )
    child_workflow_execution_canceled_event_attributes: "ChildWorkflowExecutionCanceledEventAttributes" = betterproto.message_field(
        39, group="attributes"
    )
    child_workflow_execution_timed_out_event_attributes: "ChildWorkflowExecutionTimedOutEventAttributes" = betterproto.message_field(
        40, group="attributes"
    )
    child_workflow_execution_terminated_event_attributes: "ChildWorkflowExecutionTerminatedEventAttributes" = betterproto.message_field(
        41, group="attributes"
    )
    signal_external_workflow_execution_initiated_event_attributes: "SignalExternalWorkflowExecutionInitiatedEventAttributes" = betterproto.message_field(
        42, group="attributes"
    )
    signal_external_workflow_execution_failed_event_attributes: "SignalExternalWorkflowExecutionFailedEventAttributes" = betterproto.message_field(
        43, group="attributes"
    )
    external_workflow_execution_signaled_event_attributes: "ExternalWorkflowExecutionSignaledEventAttributes" = betterproto.message_field(
        44, group="attributes"
    )
    upsert_workflow_search_attributes_event_attributes: "UpsertWorkflowSearchAttributesEventAttributes" = betterproto.message_field(
        45, group="attributes"
    )


@dataclass
class History(betterproto.Message):
    events: List["HistoryEvent"] = betterproto.message_field(1)



import datetime
import logging
import json
import inspect
from typing import List

from grpclib import GRPCError

from temporal.activity import ActivityContext, ActivityTask, complete_exceptionally, complete
from temporal.api.taskqueue.v1 import TaskQueue, TaskQueueMetadata
from temporal.converter import get_fn_args_type_hints
from temporal.retry import retry
from temporal.service_helpers import get_identity
from temporal.worker import Worker, StopRequestedException
from temporal.api.workflowservice.v1 import WorkflowServiceStub as WorkflowService, PollActivityTaskQueueRequest, \
    PollActivityTaskQueueResponse

logger = logging.getLogger(__name__)


@retry(logger=logger)
async def activity_task_loop_func(worker: Worker):
    service: WorkflowService = worker.client.service
    logger.info(f"Activity task worker started: {get_identity()}")
    try:
        while True:
            if worker.is_stop_requested():
                return
            try:
                polling_start = datetime.datetime.now()
                polling_request: PollActivityTaskQueueRequest = PollActivityTaskQueueRequest()
                polling_request.task_queue_metadata = TaskQueueMetadata()
                polling_request.task_queue_metadata.max_tasks_per_second = 200000
                polling_request.namespace = worker.namespace
                polling_request.identity = get_identity()
                polling_request.task_queue = TaskQueue()
                polling_request.task_queue.name = worker.task_queue
                task: PollActivityTaskQueueResponse
                task = await service.poll_activity_task_queue(request=polling_request)
                polling_end = datetime.datetime.now()
                logger.debug("PollActivityTaskQueue: %dms", (polling_end - polling_start).total_seconds() * 1000)
            except StopRequestedException:
                return
            except GRPCError as ex:
                logger.error("Error invoking poll_activity_task_queue: %s", ex, exc_info=True)
                continue
            task_token = task.task_token
            if not task_token:
                logger.debug("PollActivityTaskQueue has no task_token (expected): %s", task)
                continue

            logger.info(f"Request for activity: {task.activity_type.name}")
            fn = worker.activities.get(task.activity_type.name)
            if not fn:
                logger.error("Activity type not found: " + task.activity_type.name)
                continue

            args: List[object] = worker.client.data_converter.from_payloads(task.input,
                                                                            get_fn_args_type_hints(fn))

            process_start = datetime.datetime.now()
            activity_context = ActivityContext()
            activity_context.client = worker.client
            activity_context.activity_task = ActivityTask.from_poll_for_activity_task_response(task)
            activity_context.namespace = worker.namespace
            try:
                ActivityContext.set(activity_context)
                if inspect.iscoroutinefunction(fn):
                    return_value = await fn(*args)
                else:
                    raise Exception(f"Activity method {fn.__module__}.{fn.__qualname__} should be a coroutine")
                if activity_context.do_not_complete:
                    logger.info(f"Not completing activity {task.activity_type.name}({str(args)[1:-1]})")
                    continue

                logger.info(
                    f"Activity {task.activity_type.name}({str(args)[1:-1]}) returned {return_value}")

                try:
                    await complete(worker.client, task_token, return_value)
                except GRPCError as ex:
                    logger.error("Error invoking respond_activity_task_completed: %s", ex, exc_info=True)
            except Exception as ex:
                logger.error(f"Activity {task.activity_type.name} failed: {type(ex).__name__}({ex})", exc_info=True)
                try:
                    await complete_exceptionally(worker.client, task_token, ex)
                except GRPCError as ex2:
                    logger.error("Error invoking respond_activity_task_failed: %s", ex2, exc_info=True)
            finally:
                ActivityContext.set(None)
                process_end = datetime.datetime.now()
                logger.info("Process ActivityTask: %dms", (process_end - process_start).total_seconds() * 1000)
    finally:
        worker.notify_thread_stopped()
        logger.info("Activity loop ended")



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: temporal/api/enums/v1/task_queue.proto, temporal/api/enums/v1/workflow.proto, temporal/api/enums/v1/namespace.proto, temporal/api/enums/v1/failed_cause.proto, temporal/api/enums/v1/common.proto, temporal/api/enums/v1/query.proto, temporal/api/enums/v1/event_type.proto, temporal/api/enums/v1/command_type.proto
# plugin: python-betterproto
from dataclasses import dataclass

import betterproto


class TaskQueueKind(betterproto.Enum):
    TASK_QUEUE_KIND_UNSPECIFIED = 0
    TASK_QUEUE_KIND_NORMAL = 1
    TASK_QUEUE_KIND_STICKY = 2


class TaskQueueType(betterproto.Enum):
    TASK_QUEUE_TYPE_UNSPECIFIED = 0
    # Workflow type of task queue.
    TASK_QUEUE_TYPE_WORKFLOW = 1
    # Activity type of task queue.
    TASK_QUEUE_TYPE_ACTIVITY = 2


class WorkflowIdReusePolicy(betterproto.Enum):
    WORKFLOW_ID_REUSE_POLICY_UNSPECIFIED = 0
    # Allow start a workflow execution using the same workflow Id, when workflow
    # not running.
    WORKFLOW_ID_REUSE_POLICY_ALLOW_DUPLICATE = 1
    # Allow start a workflow execution using the same workflow Id, when workflow
    # not running, and the last execution close state is in [terminated,
    # cancelled, timed out, failed].
    WORKFLOW_ID_REUSE_POLICY_ALLOW_DUPLICATE_FAILED_ONLY = 2
    # Do not allow start a workflow execution using the same workflow Id at all.
    WORKFLOW_ID_REUSE_POLICY_REJECT_DUPLICATE = 3


class ParentClosePolicy(betterproto.Enum):
    PARENT_CLOSE_POLICY_UNSPECIFIED = 0
    # Terminate means terminating the child workflow.
    PARENT_CLOSE_POLICY_TERMINATE = 1
    # Abandon means not doing anything on the child workflow.
    PARENT_CLOSE_POLICY_ABANDON = 2
    # Cancel means requesting cancellation on the child workflow.
    PARENT_CLOSE_POLICY_REQUEST_CANCEL = 3


class ContinueAsNewInitiator(betterproto.Enum):
    CONTINUE_AS_NEW_INITIATOR_UNSPECIFIED = 0
    CONTINUE_AS_NEW_INITIATOR_WORKFLOW = 1
    CONTINUE_AS_NEW_INITIATOR_RETRY = 2
    CONTINUE_AS_NEW_INITIATOR_CRON_SCHEDULE = 3


class WorkflowExecutionStatus(betterproto.Enum):
    """
    (-- api-linter: core::0216::synonyms=disabled     aip.dev/not-precedent:
    There is WorkflowExecutionState already in another package. --)
    """

    WORKFLOW_EXECUTION_STATUS_UNSPECIFIED = 0
    # Value 1 is hardcoded in SQL persistence.
    WORKFLOW_EXECUTION_STATUS_RUNNING = 1
    WORKFLOW_EXECUTION_STATUS_COMPLETED = 2
    WORKFLOW_EXECUTION_STATUS_FAILED = 3
    WORKFLOW_EXECUTION_STATUS_CANCELED = 4
    WORKFLOW_EXECUTION_STATUS_TERMINATED = 5
    WORKFLOW_EXECUTION_STATUS_CONTINUED_AS_NEW = 6
    WORKFLOW_EXECUTION_STATUS_TIMED_OUT = 7


class PendingActivityState(betterproto.Enum):
    PENDING_ACTIVITY_STATE_UNSPECIFIED = 0
    PENDING_ACTIVITY_STATE_SCHEDULED = 1
    PENDING_ACTIVITY_STATE_STARTED = 2
    PENDING_ACTIVITY_STATE_CANCEL_REQUESTED = 3


class HistoryEventFilterType(betterproto.Enum):
    HISTORY_EVENT_FILTER_TYPE_UNSPECIFIED = 0
    HISTORY_EVENT_FILTER_TYPE_ALL_EVENT = 1
    HISTORY_EVENT_FILTER_TYPE_CLOSE_EVENT = 2


class RetryState(betterproto.Enum):
    RETRY_STATE_UNSPECIFIED = 0
    RETRY_STATE_IN_PROGRESS = 1
    RETRY_STATE_NON_RETRYABLE_FAILURE = 2
    RETRY_STATE_TIMEOUT = 3
    RETRY_STATE_MAXIMUM_ATTEMPTS_REACHED = 4
    RETRY_STATE_RETRY_POLICY_NOT_SET = 5
    RETRY_STATE_INTERNAL_SERVER_ERROR = 6
    RETRY_STATE_CANCEL_REQUESTED = 7


class TimeoutType(betterproto.Enum):
    TIMEOUT_TYPE_UNSPECIFIED = 0
    TIMEOUT_TYPE_START_TO_CLOSE = 1
    TIMEOUT_TYPE_SCHEDULE_TO_START = 2
    TIMEOUT_TYPE_SCHEDULE_TO_CLOSE = 3
    TIMEOUT_TYPE_HEARTBEAT = 4


class NamespaceState(betterproto.Enum):
    NAMESPACE_STATE_UNSPECIFIED = 0
    NAMESPACE_STATE_REGISTERED = 1
    NAMESPACE_STATE_DEPRECATED = 2
    NAMESPACE_STATE_DELETED = 3


class ArchivalState(betterproto.Enum):
    ARCHIVAL_STATE_UNSPECIFIED = 0
    ARCHIVAL_STATE_DISABLED = 1
    ARCHIVAL_STATE_ENABLED = 2


class WorkflowTaskFailedCause(betterproto.Enum):
    WORKFLOW_TASK_FAILED_CAUSE_UNSPECIFIED = 0
    WORKFLOW_TASK_FAILED_CAUSE_UNHANDLED_COMMAND = 1
    WORKFLOW_TASK_FAILED_CAUSE_BAD_SCHEDULE_ACTIVITY_ATTRIBUTES = 2
    WORKFLOW_TASK_FAILED_CAUSE_BAD_REQUEST_CANCEL_ACTIVITY_ATTRIBUTES = 3
    WORKFLOW_TASK_FAILED_CAUSE_BAD_START_TIMER_ATTRIBUTES = 4
    WORKFLOW_TASK_FAILED_CAUSE_BAD_CANCEL_TIMER_ATTRIBUTES = 5
    WORKFLOW_TASK_FAILED_CAUSE_BAD_RECORD_MARKER_ATTRIBUTES = 6
    WORKFLOW_TASK_FAILED_CAUSE_BAD_COMPLETE_WORKFLOW_EXECUTION_ATTRIBUTES = 7
    WORKFLOW_TASK_FAILED_CAUSE_BAD_FAIL_WORKFLOW_EXECUTION_ATTRIBUTES = 8
    WORKFLOW_TASK_FAILED_CAUSE_BAD_CANCEL_WORKFLOW_EXECUTION_ATTRIBUTES = 9
    WORKFLOW_TASK_FAILED_CAUSE_BAD_REQUEST_CANCEL_EXTERNAL_WORKFLOW_EXECUTION_ATTRIBUTES = (
        10
    )
    WORKFLOW_TASK_FAILED_CAUSE_BAD_CONTINUE_AS_NEW_ATTRIBUTES = 11
    WORKFLOW_TASK_FAILED_CAUSE_START_TIMER_DUPLICATE_ID = 12
    WORKFLOW_TASK_FAILED_CAUSE_RESET_STICKY_TASK_QUEUE = 13
    WORKFLOW_TASK_FAILED_CAUSE_WORKFLOW_WORKER_UNHANDLED_FAILURE = 14
    WORKFLOW_TASK_FAILED_CAUSE_BAD_SIGNAL_WORKFLOW_EXECUTION_ATTRIBUTES = 15
    WORKFLOW_TASK_FAILED_CAUSE_BAD_START_CHILD_EXECUTION_ATTRIBUTES = 16
    WORKFLOW_TASK_FAILED_CAUSE_FORCE_CLOSE_COMMAND = 17
    WORKFLOW_TASK_FAILED_CAUSE_FAILOVER_CLOSE_COMMAND = 18
    WORKFLOW_TASK_FAILED_CAUSE_BAD_SIGNAL_INPUT_SIZE = 19
    WORKFLOW_TASK_FAILED_CAUSE_RESET_WORKFLOW = 20
    WORKFLOW_TASK_FAILED_CAUSE_BAD_BINARY = 21
    WORKFLOW_TASK_FAILED_CAUSE_SCHEDULE_ACTIVITY_DUPLICATE_ID = 22
    WORKFLOW_TASK_FAILED_CAUSE_BAD_SEARCH_ATTRIBUTES = 23


class StartChildWorkflowExecutionFailedCause(betterproto.Enum):
    START_CHILD_WORKFLOW_EXECUTION_FAILED_CAUSE_UNSPECIFIED = 0
    START_CHILD_WORKFLOW_EXECUTION_FAILED_CAUSE_WORKFLOW_ALREADY_EXISTS = 1


class CancelExternalWorkflowExecutionFailedCause(betterproto.Enum):
    CANCEL_EXTERNAL_WORKFLOW_EXECUTION_FAILED_CAUSE_UNSPECIFIED = 0
    CANCEL_EXTERNAL_WORKFLOW_EXECUTION_FAILED_CAUSE_EXTERNAL_WORKFLOW_EXECUTION_NOT_FOUND = (
        1
    )


class SignalExternalWorkflowExecutionFailedCause(betterproto.Enum):
    SIGNAL_EXTERNAL_WORKFLOW_EXECUTION_FAILED_CAUSE_UNSPECIFIED = 0
    SIGNAL_EXTERNAL_WORKFLOW_EXECUTION_FAILED_CAUSE_EXTERNAL_WORKFLOW_EXECUTION_NOT_FOUND = (
        1
    )


class EncodingType(betterproto.Enum):
    ENCODING_TYPE_UNSPECIFIED = 0
    ENCODING_TYPE_PROTO3 = 1
    ENCODING_TYPE_JSON = 2


class IndexedValueType(betterproto.Enum):
    INDEXED_VALUE_TYPE_UNSPECIFIED = 0
    INDEXED_VALUE_TYPE_STRING = 1
    INDEXED_VALUE_TYPE_KEYWORD = 2
    INDEXED_VALUE_TYPE_INT = 3
    INDEXED_VALUE_TYPE_DOUBLE = 4
    INDEXED_VALUE_TYPE_BOOL = 5
    INDEXED_VALUE_TYPE_DATETIME = 6


class QueryResultType(betterproto.Enum):
    QUERY_RESULT_TYPE_UNSPECIFIED = 0
    QUERY_RESULT_TYPE_ANSWERED = 1
    QUERY_RESULT_TYPE_FAILED = 2


class QueryRejectCondition(betterproto.Enum):
    QUERY_REJECT_CONDITION_UNSPECIFIED = 0
    # None indicates that query should not be rejected.
    QUERY_REJECT_CONDITION_NONE = 1
    # NotOpen indicates that query should be rejected if workflow is not open.
    QUERY_REJECT_CONDITION_NOT_OPEN = 2
    # NotCompletedCleanly indicates that query should be rejected if workflow did
    # not complete cleanly.
    QUERY_REJECT_CONDITION_NOT_COMPLETED_CLEANLY = 3


class EventType(betterproto.Enum):
    """
    Whenever this list of events is changed do change the function
    shouldBufferEvent in mutableStateBuilder.go to make sure to do the correct
    event ordering.
    """

    EVENT_TYPE_UNSPECIFIED = 0
    EVENT_TYPE_WORKFLOW_EXECUTION_STARTED = 1
    EVENT_TYPE_WORKFLOW_EXECUTION_COMPLETED = 2
    EVENT_TYPE_WORKFLOW_EXECUTION_FAILED = 3
    EVENT_TYPE_WORKFLOW_EXECUTION_TIMED_OUT = 4
    EVENT_TYPE_WORKFLOW_TASK_SCHEDULED = 5
    EVENT_TYPE_WORKFLOW_TASK_STARTED = 6
    EVENT_TYPE_WORKFLOW_TASK_COMPLETED = 7
    EVENT_TYPE_WORKFLOW_TASK_TIMED_OUT = 8
    EVENT_TYPE_WORKFLOW_TASK_FAILED = 9
    EVENT_TYPE_ACTIVITY_TASK_SCHEDULED = 10
    EVENT_TYPE_ACTIVITY_TASK_STARTED = 11
    EVENT_TYPE_ACTIVITY_TASK_COMPLETED = 12
    EVENT_TYPE_ACTIVITY_TASK_FAILED = 13
    EVENT_TYPE_ACTIVITY_TASK_TIMED_OUT = 14
    EVENT_TYPE_ACTIVITY_TASK_CANCEL_REQUESTED = 15
    EVENT_TYPE_ACTIVITY_TASK_CANCELED = 16
    EVENT_TYPE_TIMER_STARTED = 17
    EVENT_TYPE_TIMER_FIRED = 18
    EVENT_TYPE_TIMER_CANCELED = 19
    EVENT_TYPE_WORKFLOW_EXECUTION_CANCEL_REQUESTED = 20
    EVENT_TYPE_WORKFLOW_EXECUTION_CANCELED = 21
    EVENT_TYPE_REQUEST_CANCEL_EXTERNAL_WORKFLOW_EXECUTION_INITIATED = 22
    EVENT_TYPE_REQUEST_CANCEL_EXTERNAL_WORKFLOW_EXECUTION_FAILED = 23
    EVENT_TYPE_EXTERNAL_WORKFLOW_EXECUTION_CANCEL_REQUESTED = 24
    EVENT_TYPE_MARKER_RECORDED = 25
    EVENT_TYPE_WORKFLOW_EXECUTION_SIGNALED = 26
    EVENT_TYPE_WORKFLOW_EXECUTION_TERMINATED = 27
    EVENT_TYPE_WORKFLOW_EXECUTION_CONTINUED_AS_NEW = 28
    EVENT_TYPE_START_CHILD_WORKFLOW_EXECUTION_INITIATED = 29
    EVENT_TYPE_START_CHILD_WORKFLOW_EXECUTION_FAILED = 30
    EVENT_TYPE_CHILD_WORKFLOW_EXECUTION_STARTED = 31
    EVENT_TYPE_CHILD_WORKFLOW_EXECUTION_COMPLETED = 32
    EVENT_TYPE_CHILD_WORKFLOW_EXECUTION_FAILED = 33
    EVENT_TYPE_CHILD_WORKFLOW_EXECUTION_CANCELED = 34
    EVENT_TYPE_CHILD_WORKFLOW_EXECUTION_TIMED_OUT = 35
    EVENT_TYPE_CHILD_WORKFLOW_EXECUTION_TERMINATED = 36
    EVENT_TYPE_SIGNAL_EXTERNAL_WORKFLOW_EXECUTION_INITIATED = 37
    EVENT_TYPE_SIGNAL_EXTERNAL_WORKFLOW_EXECUTION_FAILED = 38
    EVENT_TYPE_EXTERNAL_WORKFLOW_EXECUTION_SIGNALED = 39
    EVENT_TYPE_UPSERT_WORKFLOW_SEARCH_ATTRIBUTES = 40


class CommandType(betterproto.Enum):
    """
    Whenever this list of command types is changed do change the function
    shouldBufferEvent in mutableStateBuilder.go to make sure to do the correct
    event ordering.
    """

    COMMAND_TYPE_UNSPECIFIED = 0
    COMMAND_TYPE_SCHEDULE_ACTIVITY_TASK = 1
    COMMAND_TYPE_REQUEST_CANCEL_ACTIVITY_TASK = 2
    COMMAND_TYPE_START_TIMER = 3
    COMMAND_TYPE_COMPLETE_WORKFLOW_EXECUTION = 4
    COMMAND_TYPE_FAIL_WORKFLOW_EXECUTION = 5
    COMMAND_TYPE_CANCEL_TIMER = 6
    COMMAND_TYPE_CANCEL_WORKFLOW_EXECUTION = 7
    COMMAND_TYPE_REQUEST_CANCEL_EXTERNAL_WORKFLOW_EXECUTION = 8
    COMMAND_TYPE_RECORD_MARKER = 9
    COMMAND_TYPE_CONTINUE_AS_NEW_WORKFLOW_EXECUTION = 10
    COMMAND_TYPE_START_CHILD_WORKFLOW_EXECUTION = 11
    COMMAND_TYPE_SIGNAL_EXTERNAL_WORKFLOW_EXECUTION = 12
    COMMAND_TYPE_UPSERT_WORKFLOW_SEARCH_ATTRIBUTES = 13



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: temporal/api/command/v1/message.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import timedelta
from typing import Dict

import betterproto

from temporal.api.common import v1 as v1common
from temporal.api.enums import v1 as v1enums
from temporal.api.failure import v1 as v1failure
from temporal.api.taskqueue import v1 as v1taskqueue


@dataclass
class ScheduleActivityTaskCommandAttributes(betterproto.Message):
    activity_id: str = betterproto.string_field(1)
    activity_type: v1common.ActivityType = betterproto.message_field(2)
    namespace: str = betterproto.string_field(3)
    task_queue: v1taskqueue.TaskQueue = betterproto.message_field(4)
    header: v1common.Header = betterproto.message_field(5)
    input: v1common.Payloads = betterproto.message_field(6)
    # (-- api-linter: core::0140::prepositions=disabled     aip.dev/not-
    # precedent: "to" is used to indicate interval. --) Indicates how long the
    # caller is willing to wait for an activity completion. Limits for how long
    # retries are happening. Either this or start_to_close_timeout_seconds must
    # be specified. When not specified defaults to the workflow execution
    # timeout.
    schedule_to_close_timeout: timedelta = betterproto.message_field(7)
    # (-- api-linter: core::0140::prepositions=disabled     aip.dev/not-
    # precedent: "to" is used to indicate interval. --) Limits time an activity
    # task can stay in a task queue before a worker picks it up. This timeout is
    # always non retryable as all a retry would achieve is to put it back into
    # the same queue. Defaults to schedule_to_close_timeout_seconds or workflow
    # execution timeout if not specified.
    schedule_to_start_timeout: timedelta = betterproto.message_field(8)
    # (-- api-linter: core::0140::prepositions=disabled     aip.dev/not-
    # precedent: "to" is used to indicate interval. --) Maximum time an activity
    # is allowed to execute after a pick up by a worker. This timeout is always
    # retryable. Either this or schedule_to_close_timeout_seconds must be
    # specified.
    start_to_close_timeout: timedelta = betterproto.message_field(9)
    # Maximum time between successful worker heartbeats.
    heartbeat_timeout: timedelta = betterproto.message_field(10)
    # Activities are provided by a default retry policy controlled through the
    # service dynamic configuration. Retries are happening up to
    # schedule_to_close_timeout. To disable retries set
    # retry_policy.maximum_attempts to 1.
    retry_policy: v1common.RetryPolicy = betterproto.message_field(11)


@dataclass
class RequestCancelActivityTaskCommandAttributes(betterproto.Message):
    scheduled_event_id: int = betterproto.int64_field(1)


@dataclass
class StartTimerCommandAttributes(betterproto.Message):
    timer_id: str = betterproto.string_field(1)
    # (-- api-linter: core::0140::prepositions=disabled     aip.dev/not-
    # precedent: "to" is used to indicate interval. --)
    start_to_fire_timeout: timedelta = betterproto.message_field(2)


@dataclass
class CompleteWorkflowExecutionCommandAttributes(betterproto.Message):
    result: v1common.Payloads = betterproto.message_field(1)


@dataclass
class FailWorkflowExecutionCommandAttributes(betterproto.Message):
    failure: v1failure.Failure = betterproto.message_field(1)


@dataclass
class CancelTimerCommandAttributes(betterproto.Message):
    timer_id: str = betterproto.string_field(1)


@dataclass
class CancelWorkflowExecutionCommandAttributes(betterproto.Message):
    details: v1common.Payloads = betterproto.message_field(1)


@dataclass
class RequestCancelExternalWorkflowExecutionCommandAttributes(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    workflow_id: str = betterproto.string_field(2)
    run_id: str = betterproto.string_field(3)
    control: str = betterproto.string_field(4)
    child_workflow_only: bool = betterproto.bool_field(5)


@dataclass
class SignalExternalWorkflowExecutionCommandAttributes(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    execution: v1common.WorkflowExecution = betterproto.message_field(2)
    signal_name: str = betterproto.string_field(3)
    input: v1common.Payloads = betterproto.message_field(4)
    control: str = betterproto.string_field(5)
    child_workflow_only: bool = betterproto.bool_field(6)


@dataclass
class UpsertWorkflowSearchAttributesCommandAttributes(betterproto.Message):
    search_attributes: v1common.SearchAttributes = betterproto.message_field(1)


@dataclass
class RecordMarkerCommandAttributes(betterproto.Message):
    marker_name: str = betterproto.string_field(1)
    details: Dict[str, v1common.Payloads] = betterproto.map_field(
        2, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    header: v1common.Header = betterproto.message_field(3)
    failure: v1failure.Failure = betterproto.message_field(4)


@dataclass
class ContinueAsNewWorkflowExecutionCommandAttributes(betterproto.Message):
    workflow_type: v1common.WorkflowType = betterproto.message_field(1)
    task_queue: v1taskqueue.TaskQueue = betterproto.message_field(2)
    input: v1common.Payloads = betterproto.message_field(3)
    # workflow_execution_timeout is omitted as it shouldn'be overridden from
    # within a workflow. Timeout of a single workflow run.
    workflow_run_timeout: timedelta = betterproto.message_field(4)
    # Timeout of a single workflow task.
    workflow_task_timeout: timedelta = betterproto.message_field(5)
    backoff_start_interval: timedelta = betterproto.message_field(6)
    retry_policy: v1common.RetryPolicy = betterproto.message_field(7)
    initiator: v1enums.ContinueAsNewInitiator = betterproto.enum_field(8)
    failure: v1failure.Failure = betterproto.message_field(9)
    last_completion_result: v1common.Payloads = betterproto.message_field(10)
    cron_schedule: str = betterproto.string_field(11)
    header: v1common.Header = betterproto.message_field(12)
    memo: v1common.Memo = betterproto.message_field(13)
    search_attributes: v1common.SearchAttributes = betterproto.message_field(14)


@dataclass
class StartChildWorkflowExecutionCommandAttributes(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    workflow_id: str = betterproto.string_field(2)
    workflow_type: v1common.WorkflowType = betterproto.message_field(3)
    task_queue: v1taskqueue.TaskQueue = betterproto.message_field(4)
    input: v1common.Payloads = betterproto.message_field(5)
    # Total workflow execution timeout including retries and continue as new.
    workflow_execution_timeout: timedelta = betterproto.message_field(6)
    # Timeout of a single workflow run.
    workflow_run_timeout: timedelta = betterproto.message_field(7)
    # Timeout of a single workflow task.
    workflow_task_timeout: timedelta = betterproto.message_field(8)
    # Default: PARENT_CLOSE_POLICY_TERMINATE.
    parent_close_policy: v1enums.ParentClosePolicy = betterproto.enum_field(9)
    control: str = betterproto.string_field(10)
    # Default: WORKFLOW_ID_REUSE_POLICY_ALLOW_DUPLICATE.
    workflow_id_reuse_policy: v1enums.WorkflowIdReusePolicy = betterproto.enum_field(11)
    retry_policy: v1common.RetryPolicy = betterproto.message_field(12)
    cron_schedule: str = betterproto.string_field(13)
    header: v1common.Header = betterproto.message_field(14)
    memo: v1common.Memo = betterproto.message_field(15)
    search_attributes: v1common.SearchAttributes = betterproto.message_field(16)


@dataclass
class Command(betterproto.Message):
    command_type: v1enums.CommandType = betterproto.enum_field(1)
    schedule_activity_task_command_attributes: "ScheduleActivityTaskCommandAttributes" = betterproto.message_field(
        2, group="attributes"
    )
    start_timer_command_attributes: "StartTimerCommandAttributes" = betterproto.message_field(
        3, group="attributes"
    )
    complete_workflow_execution_command_attributes: "CompleteWorkflowExecutionCommandAttributes" = betterproto.message_field(
        4, group="attributes"
    )
    fail_workflow_execution_command_attributes: "FailWorkflowExecutionCommandAttributes" = betterproto.message_field(
        5, group="attributes"
    )
    request_cancel_activity_task_command_attributes: "RequestCancelActivityTaskCommandAttributes" = betterproto.message_field(
        6, group="attributes"
    )
    cancel_timer_command_attributes: "CancelTimerCommandAttributes" = betterproto.message_field(
        7, group="attributes"
    )
    cancel_workflow_execution_command_attributes: "CancelWorkflowExecutionCommandAttributes" = betterproto.message_field(
        8, group="attributes"
    )
    request_cancel_external_workflow_execution_command_attributes: "RequestCancelExternalWorkflowExecutionCommandAttributes" = betterproto.message_field(
        9, group="attributes"
    )
    record_marker_command_attributes: "RecordMarkerCommandAttributes" = betterproto.message_field(
        10, group="attributes"
    )
    continue_as_new_workflow_execution_command_attributes: "ContinueAsNewWorkflowExecutionCommandAttributes" = betterproto.message_field(
        11, group="attributes"
    )
    start_child_workflow_execution_command_attributes: "StartChildWorkflowExecutionCommandAttributes" = betterproto.message_field(
        12, group="attributes"
    )
    signal_external_workflow_execution_command_attributes: "SignalExternalWorkflowExecutionCommandAttributes" = betterproto.message_field(
        13, group="attributes"
    )
    upsert_workflow_search_attributes_command_attributes: "UpsertWorkflowSearchAttributesCommandAttributes" = betterproto.message_field(
        14, group="attributes"
    )



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: temporal/api/workflow/v1/message.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime, timedelta
from typing import List

import betterproto

from temporal.api.common import v1 as v1common
from temporal.api.enums import v1 as v1enums
from temporal.api.failure import v1 as v1failure
from temporal.api.taskqueue import v1 as v1taskqueue


@dataclass
class WorkflowExecutionInfo(betterproto.Message):
    execution: v1common.WorkflowExecution = betterproto.message_field(1)
    type: v1common.WorkflowType = betterproto.message_field(2)
    start_time: datetime = betterproto.message_field(3)
    close_time: datetime = betterproto.message_field(4)
    status: v1enums.WorkflowExecutionStatus = betterproto.enum_field(5)
    history_length: int = betterproto.int64_field(6)
    parent_namespace_id: str = betterproto.string_field(7)
    parent_execution: v1common.WorkflowExecution = betterproto.message_field(8)
    execution_time: datetime = betterproto.message_field(9)
    memo: v1common.Memo = betterproto.message_field(10)
    search_attributes: v1common.SearchAttributes = betterproto.message_field(11)
    auto_reset_points: "ResetPoints" = betterproto.message_field(12)
    task_queue: str = betterproto.string_field(13)


@dataclass
class WorkflowExecutionConfig(betterproto.Message):
    task_queue: v1taskqueue.TaskQueue = betterproto.message_field(1)
    workflow_execution_timeout: timedelta = betterproto.message_field(2)
    workflow_run_timeout: timedelta = betterproto.message_field(3)
    default_workflow_task_timeout: timedelta = betterproto.message_field(4)


@dataclass
class PendingActivityInfo(betterproto.Message):
    activity_id: str = betterproto.string_field(1)
    activity_type: v1common.ActivityType = betterproto.message_field(2)
    state: v1enums.PendingActivityState = betterproto.enum_field(3)
    heartbeat_details: v1common.Payloads = betterproto.message_field(4)
    last_heartbeat_time: datetime = betterproto.message_field(5)
    last_started_time: datetime = betterproto.message_field(6)
    attempt: int = betterproto.int32_field(7)
    maximum_attempts: int = betterproto.int32_field(8)
    scheduled_time: datetime = betterproto.message_field(9)
    expiration_time: datetime = betterproto.message_field(10)
    last_failure: v1failure.Failure = betterproto.message_field(11)
    last_worker_identity: str = betterproto.string_field(12)


@dataclass
class PendingChildExecutionInfo(betterproto.Message):
    workflow_id: str = betterproto.string_field(1)
    run_id: str = betterproto.string_field(2)
    workflow_type_name: str = betterproto.string_field(3)
    initiated_id: int = betterproto.int64_field(4)
    # Default: PARENT_CLOSE_POLICY_TERMINATE.
    parent_close_policy: v1enums.ParentClosePolicy = betterproto.enum_field(5)


@dataclass
class ResetPoints(betterproto.Message):
    points: List["ResetPointInfo"] = betterproto.message_field(1)


@dataclass
class ResetPointInfo(betterproto.Message):
    binary_checksum: str = betterproto.string_field(1)
    run_id: str = betterproto.string_field(2)
    first_workflow_task_completed_id: int = betterproto.int64_field(3)
    create_time: datetime = betterproto.message_field(4)
    # (-- api-linter: core::0214::resource-expiry=disabled     aip.dev/not-
    # precedent: TTL is not defined for ResetPointInfo. --) The time that the run
    # is deleted due to retention.
    expire_time: datetime = betterproto.message_field(5)
    # false if the reset point has pending childWFs/reqCancels/signalExternals.
    resettable: bool = betterproto.bool_field(6)



import contextvars
from contextvars import ContextVar
from dataclasses import dataclass
from datetime import datetime, timedelta

from typing import List, Optional

from temporal.api.common.v1 import WorkflowExecution, ActivityType, Payloads
from temporal.api.workflowservice.v1 import PollActivityTaskQueueResponse, RecordActivityTaskHeartbeatRequest, \
    RespondActivityTaskFailedRequest, RespondActivityTaskCompletedRequest
from temporal.exception_handling import serialize_exception
from temporal.exceptions import ActivityCancelledException
from temporal.service_helpers import get_identity

current_activity_context: ContextVar['ActivityContext'] = contextvars.ContextVar("current_activity_context")


@dataclass
class ActivityTask:
    @classmethod
    def from_poll_for_activity_task_response(cls, task: PollActivityTaskQueueResponse) -> 'ActivityTask':
        activity_task: 'ActivityTask' = cls()
        activity_task.task_token = task.task_token
        activity_task.workflow_execution = task.workflow_execution
        activity_task.activity_id = task.activity_id
        activity_task.activity_type = task.activity_type
        activity_task.scheduled_time = task.scheduled_time
        activity_task.schedule_to_close_timeout = task.schedule_to_close_timeout
        activity_task.start_to_close_timeout = task.start_to_close_timeout
        activity_task.heartbeat_timeout = task.heartbeat_timeout
        activity_task.attempt = task.attempt
        activity_task.heartbeat_details = task.heartbeat_details
        activity_task.workflow_namespace = task.workflow_namespace
        return activity_task

    task_token: bytes = None
    workflow_execution: WorkflowExecution = None
    activity_id: str = None
    activity_type: ActivityType = None
    scheduled_time: datetime = None
    schedule_to_close_timeout: timedelta = None
    start_to_close_timeout: timedelta = None
    heartbeat_timeout: timedelta = None
    attempt: int = None
    heartbeat_details: Payloads = None
    workflow_namespace: str = None


async def heartbeat(client: 'WorkflowClient', task_token: bytes, details: object):
    request: RecordActivityTaskHeartbeatRequest = RecordActivityTaskHeartbeatRequest()
    request.details = client.data_converter.to_payloads([details])
    request.identity = get_identity()
    request.task_token = task_token
    response = await client.service.record_activity_task_heartbeat(request=request)
    # -----
    # if error:
    #     raise error
    # -----
    if response.cancel_requested:
        raise ActivityCancelledException()


class ActivityContext:
    client: 'WorkflowClient' = None
    activity_task: ActivityTask = None
    namespace: str = None
    do_not_complete: bool = False

    @staticmethod
    def get() -> 'ActivityContext':
        return current_activity_context.get()

    @staticmethod
    def set(context: Optional['ActivityContext']):
        current_activity_context.set(context)

    # Plan for heartbeat:
    # - We will possibly implement a non-async version of this
    # - We might standardize on a background thread (using an in-memory queue) for heartbeats
    #   in which case it doesn't matter whether it's an async method or not.
    async def heartbeat(self, details: object):
        await heartbeat(self.client, self.activity_task.task_token, details)

    def get_heartbeat_details(self) -> object:
        details: Payloads = self.activity_task.heartbeat_details
        if not self.activity_task.heartbeat_details:
            return None
        payloads: List[object] = self.client.data_converter.from_payloads(details)
        return payloads[0]

    def do_not_complete_on_return(self):
        self.do_not_complete = True


class Activity:

    @staticmethod
    def get_task_token() -> bytes:
        return ActivityContext.get().activity_task.task_token

    @staticmethod
    def get_workflow_execution() -> WorkflowExecution:
        return ActivityContext.get().activity_task.workflow_execution

    @staticmethod
    def get_namespace() -> str:
        return ActivityContext.get().namespace

    @staticmethod
    def get_heartbeat_details() -> object:
        return ActivityContext.get().get_heartbeat_details()

    @staticmethod
    async def heartbeat(details: object):
        await ActivityContext.get().heartbeat(details)

    @staticmethod
    def get_activity_task() -> ActivityTask:
        return ActivityContext.get().activity_task

    @staticmethod
    def do_not_complete_on_return():
        return ActivityContext.get().do_not_complete_on_return()


@dataclass
class ActivityCompletionClient:
    client: 'WorkflowClient'

    def heartbeat(self, task_token: bytes, details: object):
        heartbeat(self.client, task_token, details)

    async def complete(self, task_token: bytes, return_value: object):
        await complete(self.client, task_token, return_value)

    async def complete_exceptionally(self, task_token: bytes, ex: Exception):
        await complete_exceptionally(self.client, task_token, ex)


async def complete_exceptionally(client: 'WorkflowClient', task_token, ex: Exception):
    respond: RespondActivityTaskFailedRequest = RespondActivityTaskFailedRequest()
    respond.task_token = task_token
    respond.identity = get_identity()
    respond.failure = serialize_exception(ex)
    await client.service.respond_activity_task_failed(request=respond)


async def complete(client: 'WorkflowClient', task_token, return_value: object):
    respond = RespondActivityTaskCompletedRequest()
    respond.task_token = task_token
    respond.result = client.data_converter.to_payloads([return_value])
    respond.identity = get_identity()
    await client.service.respond_activity_task_completed(request=respond)


from temporal.workflow import WorkflowClient



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: temporal/api/errordetails/v1/message.proto
# plugin: python-betterproto
from dataclasses import dataclass

import betterproto


@dataclass
class NotFoundFailure(betterproto.Message):
    current_cluster: str = betterproto.string_field(1)
    active_cluster: str = betterproto.string_field(2)


@dataclass
class WorkflowExecutionAlreadyStartedFailure(betterproto.Message):
    start_request_id: str = betterproto.string_field(1)
    run_id: str = betterproto.string_field(2)


@dataclass
class NamespaceNotActiveFailure(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    current_cluster: str = betterproto.string_field(2)
    active_cluster: str = betterproto.string_field(3)


@dataclass
class ClientVersionNotSupportedFailure(betterproto.Message):
    client_version: str = betterproto.string_field(1)
    client_impl: str = betterproto.string_field(2)
    supported_versions: str = betterproto.string_field(3)


@dataclass
class FeatureVersionNotSupportedFailure(betterproto.Message):
    feature: str = betterproto.string_field(1)
    feature_version: str = betterproto.string_field(2)
    supported_versions: str = betterproto.string_field(3)


@dataclass
class NamespaceAlreadyExistsFailure(betterproto.Message):
    pass


@dataclass
class CancellationAlreadyRequestedFailure(betterproto.Message):
    pass


@dataclass
class QueryFailedFailure(betterproto.Message):
    pass



import os
import socket

from grpclib.client import Channel

from temporal.api.workflowservice.v1 import WorkflowServiceStub


def create_workflow_service(host: str, port: int, timeout: float) -> WorkflowServiceStub:
    channel = Channel(host=host, port=port)
    return WorkflowServiceStub(channel, timeout=timeout)


def get_identity():
    return "%d@%s" % (os.getpid(), socket.gethostname())




# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: temporal/api/taskqueue/v1/message.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime, timedelta
from typing import Optional

import betterproto

from temporal.api.enums import v1 as v1enums


@dataclass
class TaskQueue(betterproto.Message):
    name: str = betterproto.string_field(1)
    # Default: TASK_QUEUE_KIND_NORMAL.
    kind: v1enums.TaskQueueKind = betterproto.enum_field(2)


@dataclass
class TaskQueueMetadata(betterproto.Message):
    max_tasks_per_second: Optional[float] = betterproto.message_field(
        1, wraps=betterproto.TYPE_DOUBLE
    )


@dataclass
class TaskQueueStatus(betterproto.Message):
    backlog_count_hint: int = betterproto.int64_field(1)
    read_level: int = betterproto.int64_field(2)
    ack_level: int = betterproto.int64_field(3)
    rate_per_second: float = betterproto.double_field(4)
    task_id_block: "TaskIdBlock" = betterproto.message_field(5)


@dataclass
class TaskIdBlock(betterproto.Message):
    start_id: int = betterproto.int64_field(1)
    end_id: int = betterproto.int64_field(2)


@dataclass
class TaskQueuePartitionMetadata(betterproto.Message):
    key: str = betterproto.string_field(1)
    owner_host_name: str = betterproto.string_field(2)


@dataclass
class PollerInfo(betterproto.Message):
    # Unix Nano
    last_access_time: datetime = betterproto.message_field(1)
    identity: str = betterproto.string_field(2)
    rate_per_second: float = betterproto.double_field(3)


@dataclass
class StickyExecutionAttributes(betterproto.Message):
    worker_task_queue: "TaskQueue" = betterproto.message_field(1)
    # (-- api-linter: core::0140::prepositions=disabled     aip.dev/not-
    # precedent: "to" is used to indicate interval. --)
    schedule_to_start_timeout: timedelta = betterproto.message_field(2)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: temporal/api/replication/v1/message.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import betterproto


@dataclass
class ClusterReplicationConfig(betterproto.Message):
    cluster_name: str = betterproto.string_field(1)


@dataclass
class NamespaceReplicationConfig(betterproto.Message):
    active_cluster_name: str = betterproto.string_field(1)
    clusters: List["ClusterReplicationConfig"] = betterproto.message_field(2)



import copy
from asyncio import Future
from typing import List, Union

from temporal.activity_method import ExecuteActivityParameters
from temporal.decision_loop import ActivityFuture


class Async:
    @staticmethod
    def function(method, *args) -> ActivityFuture:
        return Async.function_with_self(method, method.__self__, *args)

    @staticmethod
    def function_with_self(method, self, *args):
        assert self._decision_context
        assert method._execute_parameters
        parameters: ExecuteActivityParameters = copy.deepcopy(method._execute_parameters)
        return Async.call(self, parameters, args)

    @staticmethod
    def call(self, parameters, args: List[object]):
        if hasattr(self, "_activity_options") and self._activity_options:
            self._activity_options.fill_execute_activity_parameters(parameters)
        if self._retry_parameters:
            parameters.retry_parameters = self._retry_parameters
        from temporal.decision_loop import DecisionContext
        decision_context: DecisionContext = self._decision_context
        parameters.input = decision_context.decider.worker.client.data_converter.to_payloads(args)
        return decision_context.schedule_activity_task(parameters=parameters)

    @staticmethod
    async def any_of(futures: List[Union[ActivityFuture, Future]], timeout_seconds=0):
        done, pending = [], []

        def condition():
            done[:] = []
            pending[:] = []
            for f in futures:
                if f.done():
                    done.append(f)
                else:
                    pending.append(f)
            if done:
                return True
            else:
                return False

        await Workflow.await_till(condition, timeout_seconds=timeout_seconds)
        return done, pending

    @staticmethod
    async def all_of(futures: List[Union[ActivityFuture, Future]], timeout_seconds=0):

        def condition():
            for f in futures:
                if not f.done():
                    return False
            return True

        await Workflow.await_till(condition, timeout_seconds=timeout_seconds)


from temporal.workflow import Workflow



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: temporal/api/workflowservice/v1/request_response.proto, temporal/api/workflowservice/v1/service.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime, timedelta
from typing import Dict, List, Optional

import betterproto
import grpclib

from temporal.api.command import v1 as v1command
from temporal.api.common import v1 as v1common
from temporal.api.enums import v1 as v1enums
from temporal.api.failure import v1 as v1failure
from temporal.api.filter import v1 as v1filter
from temporal.api.history import v1 as v1history
from temporal.api.namespace import v1 as v1namespace
from temporal.api.query import v1 as v1query
from temporal.api.replication import v1 as v1replication
from temporal.api.taskqueue import v1 as v1taskqueue
from temporal.api.version import v1 as v1version
from temporal.api.workflow import v1 as v1workflow


@dataclass
class RegisterNamespaceRequest(betterproto.Message):
    name: str = betterproto.string_field(1)
    description: str = betterproto.string_field(2)
    owner_email: str = betterproto.string_field(3)
    workflow_execution_retention_period: timedelta = betterproto.message_field(4)
    clusters: List[v1replication.ClusterReplicationConfig] = betterproto.message_field(
        5
    )
    active_cluster_name: str = betterproto.string_field(6)
    # A key-value map for any customized purpose.
    data: Dict[str, str] = betterproto.map_field(
        7, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )
    security_token: str = betterproto.string_field(8)
    is_global_namespace: bool = betterproto.bool_field(9)
    # If unspecified (ARCHIVAL_STATE_UNSPECIFIED) then default server
    # configuration is used.
    history_archival_state: v1enums.ArchivalState = betterproto.enum_field(10)
    history_archival_uri: str = betterproto.string_field(11)
    # If unspecified (ARCHIVAL_STATE_UNSPECIFIED) then default server
    # configuration is used.
    visibility_archival_state: v1enums.ArchivalState = betterproto.enum_field(12)
    visibility_archival_uri: str = betterproto.string_field(13)


@dataclass
class RegisterNamespaceResponse(betterproto.Message):
    pass


@dataclass
class ListNamespacesRequest(betterproto.Message):
    page_size: int = betterproto.int32_field(1)
    next_page_token: bytes = betterproto.bytes_field(2)


@dataclass
class ListNamespacesResponse(betterproto.Message):
    namespaces: List["DescribeNamespaceResponse"] = betterproto.message_field(1)
    next_page_token: bytes = betterproto.bytes_field(2)


@dataclass
class DescribeNamespaceRequest(betterproto.Message):
    name: str = betterproto.string_field(1)
    id: str = betterproto.string_field(2)


@dataclass
class DescribeNamespaceResponse(betterproto.Message):
    namespace_info: v1namespace.NamespaceInfo = betterproto.message_field(1)
    config: v1namespace.NamespaceConfig = betterproto.message_field(2)
    replication_config: v1replication.NamespaceReplicationConfig = betterproto.message_field(
        3
    )
    failover_version: int = betterproto.int64_field(4)
    is_global_namespace: bool = betterproto.bool_field(5)


@dataclass
class UpdateNamespaceRequest(betterproto.Message):
    """
    (-- api-linter: core::0134::request-mask-required=disabled     aip.dev/not-
    precedent: UpdateNamespace RPC doesn't follow Google API format. --) (--
    api-linter: core::0134::request-resource-required=disabled     aip.dev/not-
    precedent: UpdateNamespace RPC doesn't follow Google API format. --)
    """

    name: str = betterproto.string_field(1)
    update_info: v1namespace.UpdateNamespaceInfo = betterproto.message_field(2)
    config: v1namespace.NamespaceConfig = betterproto.message_field(3)
    replication_config: v1replication.NamespaceReplicationConfig = betterproto.message_field(
        4
    )
    security_token: str = betterproto.string_field(5)
    delete_bad_binary: str = betterproto.string_field(6)


@dataclass
class UpdateNamespaceResponse(betterproto.Message):
    namespace_info: v1namespace.NamespaceInfo = betterproto.message_field(1)
    config: v1namespace.NamespaceConfig = betterproto.message_field(2)
    replication_config: v1replication.NamespaceReplicationConfig = betterproto.message_field(
        3
    )
    failover_version: int = betterproto.int64_field(4)
    is_global_namespace: bool = betterproto.bool_field(5)


@dataclass
class DeprecateNamespaceRequest(betterproto.Message):
    name: str = betterproto.string_field(1)
    security_token: str = betterproto.string_field(2)


@dataclass
class DeprecateNamespaceResponse(betterproto.Message):
    pass


@dataclass
class StartWorkflowExecutionRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    workflow_id: str = betterproto.string_field(2)
    workflow_type: v1common.WorkflowType = betterproto.message_field(3)
    task_queue: v1taskqueue.TaskQueue = betterproto.message_field(4)
    input: v1common.Payloads = betterproto.message_field(5)
    # Total workflow execution timeout including retries and continue as new.
    workflow_execution_timeout: timedelta = betterproto.message_field(6)
    # Timeout of a single workflow run.
    workflow_run_timeout: timedelta = betterproto.message_field(7)
    # Timeout of a single workflow task.
    workflow_task_timeout: timedelta = betterproto.message_field(8)
    identity: str = betterproto.string_field(9)
    request_id: str = betterproto.string_field(10)
    # Default: WORKFLOW_ID_REUSE_POLICY_ALLOW_DUPLICATE.
    workflow_id_reuse_policy: v1enums.WorkflowIdReusePolicy = betterproto.enum_field(11)
    # Retries up to workflow_execution_timeout_seconds.
    retry_policy: v1common.RetryPolicy = betterproto.message_field(12)
    cron_schedule: str = betterproto.string_field(13)
    memo: v1common.Memo = betterproto.message_field(14)
    search_attributes: v1common.SearchAttributes = betterproto.message_field(15)
    header: v1common.Header = betterproto.message_field(16)


@dataclass
class StartWorkflowExecutionResponse(betterproto.Message):
    run_id: str = betterproto.string_field(1)


@dataclass
class GetWorkflowExecutionHistoryRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    execution: v1common.WorkflowExecution = betterproto.message_field(2)
    maximum_page_size: int = betterproto.int32_field(3)
    next_page_token: bytes = betterproto.bytes_field(4)
    wait_new_event: bool = betterproto.bool_field(5)
    # Default: HISTORY_EVENT_FILTER_TYPE_ALL_EVENT.
    history_event_filter_type: v1enums.HistoryEventFilterType = betterproto.enum_field(
        6
    )
    skip_archival: bool = betterproto.bool_field(7)


@dataclass
class GetWorkflowExecutionHistoryResponse(betterproto.Message):
    history: v1history.History = betterproto.message_field(1)
    raw_history: List[v1common.DataBlob] = betterproto.message_field(2)
    next_page_token: bytes = betterproto.bytes_field(3)
    archived: bool = betterproto.bool_field(4)


@dataclass
class PollWorkflowTaskQueueRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    task_queue: v1taskqueue.TaskQueue = betterproto.message_field(2)
    identity: str = betterproto.string_field(3)
    binary_checksum: str = betterproto.string_field(4)


@dataclass
class PollWorkflowTaskQueueResponse(betterproto.Message):
    task_token: bytes = betterproto.bytes_field(1)
    workflow_execution: v1common.WorkflowExecution = betterproto.message_field(2)
    workflow_type: v1common.WorkflowType = betterproto.message_field(3)
    previous_started_event_id: int = betterproto.int64_field(4)
    started_event_id: int = betterproto.int64_field(5)
    attempt: int = betterproto.int32_field(6)
    backlog_count_hint: int = betterproto.int64_field(7)
    history: v1history.History = betterproto.message_field(8)
    next_page_token: bytes = betterproto.bytes_field(9)
    query: v1query.WorkflowQuery = betterproto.message_field(10)
    workflow_execution_task_queue: v1taskqueue.TaskQueue = betterproto.message_field(11)
    scheduled_time: datetime = betterproto.message_field(12)
    started_time: datetime = betterproto.message_field(13)
    queries: Dict[str, v1query.WorkflowQuery] = betterproto.map_field(
        14, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )


@dataclass
class RespondWorkflowTaskCompletedRequest(betterproto.Message):
    task_token: bytes = betterproto.bytes_field(1)
    commands: List[v1command.Command] = betterproto.message_field(2)
    identity: str = betterproto.string_field(3)
    sticky_attributes: v1taskqueue.StickyExecutionAttributes = betterproto.message_field(
        4
    )
    return_new_workflow_task: bool = betterproto.bool_field(5)
    force_create_new_workflow_task: bool = betterproto.bool_field(6)
    binary_checksum: str = betterproto.string_field(7)
    query_results: Dict[str, v1query.WorkflowQueryResult] = betterproto.map_field(
        8, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )


@dataclass
class RespondWorkflowTaskCompletedResponse(betterproto.Message):
    workflow_task: "PollWorkflowTaskQueueResponse" = betterproto.message_field(1)


@dataclass
class RespondWorkflowTaskFailedRequest(betterproto.Message):
    task_token: bytes = betterproto.bytes_field(1)
    cause: v1enums.WorkflowTaskFailedCause = betterproto.enum_field(2)
    failure: v1failure.Failure = betterproto.message_field(3)
    identity: str = betterproto.string_field(4)
    binary_checksum: str = betterproto.string_field(5)


@dataclass
class RespondWorkflowTaskFailedResponse(betterproto.Message):
    pass


@dataclass
class PollActivityTaskQueueRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    task_queue: v1taskqueue.TaskQueue = betterproto.message_field(2)
    identity: str = betterproto.string_field(3)
    task_queue_metadata: v1taskqueue.TaskQueueMetadata = betterproto.message_field(4)


@dataclass
class PollActivityTaskQueueResponse(betterproto.Message):
    task_token: bytes = betterproto.bytes_field(1)
    workflow_namespace: str = betterproto.string_field(2)
    workflow_type: v1common.WorkflowType = betterproto.message_field(3)
    workflow_execution: v1common.WorkflowExecution = betterproto.message_field(4)
    activity_type: v1common.ActivityType = betterproto.message_field(5)
    activity_id: str = betterproto.string_field(6)
    header: v1common.Header = betterproto.message_field(7)
    input: v1common.Payloads = betterproto.message_field(8)
    heartbeat_details: v1common.Payloads = betterproto.message_field(9)
    scheduled_time: datetime = betterproto.message_field(10)
    current_attempt_scheduled_time: datetime = betterproto.message_field(11)
    started_time: datetime = betterproto.message_field(12)
    attempt: int = betterproto.int32_field(13)
    # (-- api-linter: core::0140::prepositions=disabled     aip.dev/not-
    # precedent: "to" is used to indicate interval. --)
    schedule_to_close_timeout: timedelta = betterproto.message_field(14)
    # (-- api-linter: core::0140::prepositions=disabled     aip.dev/not-
    # precedent: "to" is used to indicate interval. --)
    start_to_close_timeout: timedelta = betterproto.message_field(15)
    heartbeat_timeout: timedelta = betterproto.message_field(16)
    # This is an actual retry policy the service uses. It can be different from
    # the one provided (or not) during activity scheduling as the service can
    # override the provided one in case its values are not specified or exceed
    # configured system limits.
    retry_policy: v1common.RetryPolicy = betterproto.message_field(17)


@dataclass
class RecordActivityTaskHeartbeatRequest(betterproto.Message):
    task_token: bytes = betterproto.bytes_field(1)
    details: v1common.Payloads = betterproto.message_field(2)
    identity: str = betterproto.string_field(3)


@dataclass
class RecordActivityTaskHeartbeatResponse(betterproto.Message):
    cancel_requested: bool = betterproto.bool_field(1)


@dataclass
class RecordActivityTaskHeartbeatByIdRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    workflow_id: str = betterproto.string_field(2)
    run_id: str = betterproto.string_field(3)
    activity_id: str = betterproto.string_field(4)
    details: v1common.Payloads = betterproto.message_field(5)
    identity: str = betterproto.string_field(6)


@dataclass
class RecordActivityTaskHeartbeatByIdResponse(betterproto.Message):
    cancel_requested: bool = betterproto.bool_field(1)


@dataclass
class RespondActivityTaskCompletedRequest(betterproto.Message):
    task_token: bytes = betterproto.bytes_field(1)
    result: v1common.Payloads = betterproto.message_field(2)
    identity: str = betterproto.string_field(3)


@dataclass
class RespondActivityTaskCompletedResponse(betterproto.Message):
    pass


@dataclass
class RespondActivityTaskCompletedByIdRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    workflow_id: str = betterproto.string_field(2)
    run_id: str = betterproto.string_field(3)
    activity_id: str = betterproto.string_field(4)
    result: v1common.Payloads = betterproto.message_field(5)
    identity: str = betterproto.string_field(6)


@dataclass
class RespondActivityTaskCompletedByIdResponse(betterproto.Message):
    pass


@dataclass
class RespondActivityTaskFailedRequest(betterproto.Message):
    task_token: bytes = betterproto.bytes_field(1)
    failure: v1failure.Failure = betterproto.message_field(2)
    identity: str = betterproto.string_field(3)


@dataclass
class RespondActivityTaskFailedResponse(betterproto.Message):
    pass


@dataclass
class RespondActivityTaskFailedByIdRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    workflow_id: str = betterproto.string_field(2)
    run_id: str = betterproto.string_field(3)
    activity_id: str = betterproto.string_field(4)
    failure: v1failure.Failure = betterproto.message_field(5)
    identity: str = betterproto.string_field(6)


@dataclass
class RespondActivityTaskFailedByIdResponse(betterproto.Message):
    pass


@dataclass
class RespondActivityTaskCanceledRequest(betterproto.Message):
    task_token: bytes = betterproto.bytes_field(1)
    details: v1common.Payloads = betterproto.message_field(2)
    identity: str = betterproto.string_field(3)


@dataclass
class RespondActivityTaskCanceledResponse(betterproto.Message):
    pass


@dataclass
class RespondActivityTaskCanceledByIdRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    workflow_id: str = betterproto.string_field(2)
    run_id: str = betterproto.string_field(3)
    activity_id: str = betterproto.string_field(4)
    details: v1common.Payloads = betterproto.message_field(5)
    identity: str = betterproto.string_field(6)


@dataclass
class RespondActivityTaskCanceledByIdResponse(betterproto.Message):
    pass


@dataclass
class RequestCancelWorkflowExecutionRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    workflow_execution: v1common.WorkflowExecution = betterproto.message_field(2)
    identity: str = betterproto.string_field(3)
    request_id: str = betterproto.string_field(4)


@dataclass
class RequestCancelWorkflowExecutionResponse(betterproto.Message):
    pass


@dataclass
class SignalWorkflowExecutionRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    workflow_execution: v1common.WorkflowExecution = betterproto.message_field(2)
    signal_name: str = betterproto.string_field(3)
    input: v1common.Payloads = betterproto.message_field(4)
    identity: str = betterproto.string_field(5)
    request_id: str = betterproto.string_field(6)
    control: str = betterproto.string_field(7)


@dataclass
class SignalWorkflowExecutionResponse(betterproto.Message):
    pass


@dataclass
class SignalWithStartWorkflowExecutionRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    workflow_id: str = betterproto.string_field(2)
    workflow_type: v1common.WorkflowType = betterproto.message_field(3)
    task_queue: v1taskqueue.TaskQueue = betterproto.message_field(4)
    input: v1common.Payloads = betterproto.message_field(5)
    # Total workflow execution timeout including retries and continue as new
    workflow_execution_timeout: timedelta = betterproto.message_field(6)
    # Timeout of a single workflow run
    workflow_run_timeout: timedelta = betterproto.message_field(7)
    # Timeout of a single workflow task
    workflow_task_timeout: timedelta = betterproto.message_field(8)
    identity: str = betterproto.string_field(9)
    request_id: str = betterproto.string_field(10)
    workflow_id_reuse_policy: v1enums.WorkflowIdReusePolicy = betterproto.enum_field(11)
    signal_name: str = betterproto.string_field(12)
    signal_input: v1common.Payloads = betterproto.message_field(13)
    control: str = betterproto.string_field(14)
    # Default: WORKFLOW_ID_REUSE_POLICY_ALLOW_DUPLICATE.
    retry_policy: v1common.RetryPolicy = betterproto.message_field(15)
    cron_schedule: str = betterproto.string_field(16)
    memo: v1common.Memo = betterproto.message_field(17)
    search_attributes: v1common.SearchAttributes = betterproto.message_field(18)
    header: v1common.Header = betterproto.message_field(19)


@dataclass
class SignalWithStartWorkflowExecutionResponse(betterproto.Message):
    run_id: str = betterproto.string_field(1)


@dataclass
class ResetWorkflowExecutionRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    workflow_execution: v1common.WorkflowExecution = betterproto.message_field(2)
    reason: str = betterproto.string_field(3)
    workflow_task_finish_event_id: int = betterproto.int64_field(4)
    request_id: str = betterproto.string_field(5)


@dataclass
class ResetWorkflowExecutionResponse(betterproto.Message):
    run_id: str = betterproto.string_field(1)


@dataclass
class TerminateWorkflowExecutionRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    workflow_execution: v1common.WorkflowExecution = betterproto.message_field(2)
    reason: str = betterproto.string_field(3)
    details: v1common.Payloads = betterproto.message_field(4)
    identity: str = betterproto.string_field(5)


@dataclass
class TerminateWorkflowExecutionResponse(betterproto.Message):
    pass


@dataclass
class ListOpenWorkflowExecutionsRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    maximum_page_size: int = betterproto.int32_field(2)
    next_page_token: bytes = betterproto.bytes_field(3)
    start_time_filter: v1filter.StartTimeFilter = betterproto.message_field(4)
    execution_filter: v1filter.WorkflowExecutionFilter = betterproto.message_field(
        5, group="filters"
    )
    type_filter: v1filter.WorkflowTypeFilter = betterproto.message_field(
        6, group="filters"
    )


@dataclass
class ListOpenWorkflowExecutionsResponse(betterproto.Message):
    executions: List[v1workflow.WorkflowExecutionInfo] = betterproto.message_field(1)
    next_page_token: bytes = betterproto.bytes_field(2)


@dataclass
class ListClosedWorkflowExecutionsRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    maximum_page_size: int = betterproto.int32_field(2)
    next_page_token: bytes = betterproto.bytes_field(3)
    start_time_filter: v1filter.StartTimeFilter = betterproto.message_field(4)
    execution_filter: v1filter.WorkflowExecutionFilter = betterproto.message_field(
        5, group="filters"
    )
    type_filter: v1filter.WorkflowTypeFilter = betterproto.message_field(
        6, group="filters"
    )
    status_filter: v1filter.StatusFilter = betterproto.message_field(7, group="filters")


@dataclass
class ListClosedWorkflowExecutionsResponse(betterproto.Message):
    executions: List[v1workflow.WorkflowExecutionInfo] = betterproto.message_field(1)
    next_page_token: bytes = betterproto.bytes_field(2)


@dataclass
class ListWorkflowExecutionsRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    page_size: int = betterproto.int32_field(2)
    next_page_token: bytes = betterproto.bytes_field(3)
    query: str = betterproto.string_field(4)


@dataclass
class ListWorkflowExecutionsResponse(betterproto.Message):
    executions: List[v1workflow.WorkflowExecutionInfo] = betterproto.message_field(1)
    next_page_token: bytes = betterproto.bytes_field(2)


@dataclass
class ListArchivedWorkflowExecutionsRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    page_size: int = betterproto.int32_field(2)
    next_page_token: bytes = betterproto.bytes_field(3)
    query: str = betterproto.string_field(4)


@dataclass
class ListArchivedWorkflowExecutionsResponse(betterproto.Message):
    executions: List[v1workflow.WorkflowExecutionInfo] = betterproto.message_field(1)
    next_page_token: bytes = betterproto.bytes_field(2)


@dataclass
class ScanWorkflowExecutionsRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    page_size: int = betterproto.int32_field(2)
    next_page_token: bytes = betterproto.bytes_field(3)
    query: str = betterproto.string_field(4)


@dataclass
class ScanWorkflowExecutionsResponse(betterproto.Message):
    executions: List[v1workflow.WorkflowExecutionInfo] = betterproto.message_field(1)
    next_page_token: bytes = betterproto.bytes_field(2)


@dataclass
class CountWorkflowExecutionsRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    query: str = betterproto.string_field(2)


@dataclass
class CountWorkflowExecutionsResponse(betterproto.Message):
    count: int = betterproto.int64_field(1)


@dataclass
class GetSearchAttributesRequest(betterproto.Message):
    pass


@dataclass
class GetSearchAttributesResponse(betterproto.Message):
    keys: Dict[str, v1enums.IndexedValueType] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_ENUM
    )


@dataclass
class RespondQueryTaskCompletedRequest(betterproto.Message):
    """TODO:  deprecated APIs"""

    task_token: bytes = betterproto.bytes_field(1)
    completed_type: v1enums.QueryResultType = betterproto.enum_field(2)
    query_result: v1common.Payloads = betterproto.message_field(3)
    error_message: str = betterproto.string_field(4)
    worker_version_info: v1version.WorkerVersionInfo = betterproto.message_field(5)


@dataclass
class RespondQueryTaskCompletedResponse(betterproto.Message):
    pass


@dataclass
class ResetStickyTaskQueueRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    execution: v1common.WorkflowExecution = betterproto.message_field(2)


@dataclass
class ResetStickyTaskQueueResponse(betterproto.Message):
    pass


@dataclass
class QueryWorkflowRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    execution: v1common.WorkflowExecution = betterproto.message_field(2)
    query: v1query.WorkflowQuery = betterproto.message_field(3)
    # QueryRejectCondition can used to reject the query if workflow state does
    # not satisfy condition. Default: QUERY_REJECT_CONDITION_NONE.
    query_reject_condition: v1enums.QueryRejectCondition = betterproto.enum_field(4)


@dataclass
class QueryWorkflowResponse(betterproto.Message):
    query_result: v1common.Payloads = betterproto.message_field(1)
    query_rejected: v1query.QueryRejected = betterproto.message_field(2)


@dataclass
class DescribeWorkflowExecutionRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    execution: v1common.WorkflowExecution = betterproto.message_field(2)


@dataclass
class DescribeWorkflowExecutionResponse(betterproto.Message):
    execution_config: v1workflow.WorkflowExecutionConfig = betterproto.message_field(1)
    workflow_execution_info: v1workflow.WorkflowExecutionInfo = betterproto.message_field(
        2
    )
    pending_activities: List[
        v1workflow.PendingActivityInfo
    ] = betterproto.message_field(3)
    pending_children: List[
        v1workflow.PendingChildExecutionInfo
    ] = betterproto.message_field(4)


@dataclass
class DescribeTaskQueueRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    task_queue: v1taskqueue.TaskQueue = betterproto.message_field(2)
    task_queue_type: v1enums.TaskQueueType = betterproto.enum_field(3)
    include_task_queue_status: bool = betterproto.bool_field(4)


@dataclass
class DescribeTaskQueueResponse(betterproto.Message):
    pollers: List[v1taskqueue.PollerInfo] = betterproto.message_field(1)
    task_queue_status: v1taskqueue.TaskQueueStatus = betterproto.message_field(2)


@dataclass
class GetClusterInfoRequest(betterproto.Message):
    pass


@dataclass
class GetClusterInfoResponse(betterproto.Message):
    """GetClusterInfoResponse contains information about Temporal cluster"""

    supported_sdk_versions: v1version.SupportedSDKVersions = betterproto.message_field(
        1
    )


@dataclass
class ListTaskQueuePartitionsRequest(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    task_queue: v1taskqueue.TaskQueue = betterproto.message_field(2)


@dataclass
class ListTaskQueuePartitionsResponse(betterproto.Message):
    activity_task_queue_partitions: List[
        v1taskqueue.TaskQueuePartitionMetadata
    ] = betterproto.message_field(1)
    workflow_task_queue_partitions: List[
        v1taskqueue.TaskQueuePartitionMetadata
    ] = betterproto.message_field(2)


class WorkflowServiceStub(betterproto.ServiceStub):
    """
    WorkflowService API is exposed to provide support for long running
    applications.  Application is expected to call StartWorkflowExecution to
    create an instance for each instance of long running workflow.  Such
    applications are expected to have a worker which regularly polls for
    WorkflowTask and ActivityTask from the WorkflowService.  For each
    WorkflowTask, application is expected to process the history of events for
    that session and respond back with next commands.  For each ActivityTask,
    application is expected to execute the actual logic for that task and
    respond back with completion or failure.  Worker is expected to regularly
    heartbeat while activity task is running.
    """

    async def register_namespace(
        self, *, request: RegisterNamespaceRequest
    ) -> RegisterNamespaceResponse:
        """
        RegisterNamespace creates a new namespace which can be used as a
        container for all resources.  Namespace is a top level entity within
        Temporal, used as a container for all resources like workflow
        executions, task queues, etc.  Namespace acts as a sandbox and provides
        isolation for all resources within the namespace.  All resources
        belongs to exactly one namespace.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/RegisterNamespace",
            request,
            RegisterNamespaceResponse,
        )

    async def describe_namespace(
        self, *, request: DescribeNamespaceRequest
    ) -> DescribeNamespaceResponse:
        """
        DescribeNamespace returns the information and configuration for a
        registered namespace.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/DescribeNamespace",
            request,
            DescribeNamespaceResponse,
        )

    async def list_namespaces(
        self, *, request: ListNamespacesRequest
    ) -> ListNamespacesResponse:
        """
        ListNamespaces returns the information and configuration for all
        namespaces.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/ListNamespaces",
            request,
            ListNamespacesResponse,
        )

    async def update_namespace(
        self, *, request: UpdateNamespaceRequest
    ) -> UpdateNamespaceResponse:
        """
        (-- api-linter: core::0134::method-signature=disabled     aip.dev/not-
        precedent: UpdateNamespace RPC doesn't follow Google API format. --)
        (-- api-linter: core::0134::response-message-name=disabled
        aip.dev/not-precedent: UpdateNamespace RPC doesn't follow Google API
        format. --) UpdateNamespace is used to update the information and
        configuration for a registered namespace.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/UpdateNamespace",
            request,
            UpdateNamespaceResponse,
        )

    async def deprecate_namespace(
        self, *, request: DeprecateNamespaceRequest
    ) -> DeprecateNamespaceResponse:
        """
        DeprecateNamespace is used to update state of a registered namespace to
        DEPRECATED.  Once the namespace is deprecated it cannot be used to
        start new workflow executions.  Existing workflow executions will
        continue to run on deprecated namespaces.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/DeprecateNamespace",
            request,
            DeprecateNamespaceResponse,
        )

    async def start_workflow_execution(
        self, *, request: StartWorkflowExecutionRequest
    ) -> StartWorkflowExecutionResponse:
        """
        StartWorkflowExecution starts a new long running workflow instance.  It
        will create the instance with 'WorkflowExecutionStarted' event in
        history and also schedule the first WorkflowTask for the worker to make
        the first command for this instance.  It will return
        'WorkflowExecutionAlreadyStartedFailure', if an instance already exists
        with same workflowId.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/StartWorkflowExecution",
            request,
            StartWorkflowExecutionResponse,
        )

    async def get_workflow_execution_history(
        self, *, request: GetWorkflowExecutionHistoryRequest
    ) -> GetWorkflowExecutionHistoryResponse:
        """
        GetWorkflowExecutionHistory returns the history of specified workflow
        execution.  It fails with 'NotFoundFailure' if specified workflow
        execution in unknown to the service.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/GetWorkflowExecutionHistory",
            request,
            GetWorkflowExecutionHistoryResponse,
        )

    async def poll_workflow_task_queue(
        self, *, request: PollWorkflowTaskQueueRequest
    ) -> PollWorkflowTaskQueueResponse:
        """
        PollWorkflowTaskQueue is called by application worker to process
        WorkflowTask from a specific task queue.  A WorkflowTask is dispatched
        to callers for active workflow executions, with pending workflow tasks.
        Application is then expected to call 'RespondWorkflowTaskCompleted' API
        when it is done processing the WorkflowTask. It will also create a
        'WorkflowTaskStarted' event in the history for that session before
        handing off WorkflowTask to application worker.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/PollWorkflowTaskQueue",
            request,
            PollWorkflowTaskQueueResponse,
        )

    async def respond_workflow_task_completed(
        self, *, request: RespondWorkflowTaskCompletedRequest
    ) -> RespondWorkflowTaskCompletedResponse:
        """
        RespondWorkflowTaskCompleted is called by application worker to
        complete a WorkflowTask handed as a result of 'PollWorkflowTaskQueue'
        API call.  Completing a WorkflowTask will result in new events for the
        workflow execution and potentially new ActivityTask being created for
        corresponding commands.  It will also create a WorkflowTaskCompleted
        event in the history for that session.  Use the 'taskToken' provided as
        response of PollWorkflowTaskQueue API call for completing the
        WorkflowTask. The response could contain a new workflow task if there
        is one or if the request asking for one.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/RespondWorkflowTaskCompleted",
            request,
            RespondWorkflowTaskCompletedResponse,
        )

    async def respond_workflow_task_failed(
        self, *, request: RespondWorkflowTaskFailedRequest
    ) -> RespondWorkflowTaskFailedResponse:
        """
        RespondWorkflowTaskFailed is called by application worker to indicate
        failure.  This results in WorkflowTaskFailedEvent written to the
        history and a new WorkflowTask created.  This API can be used by client
        to either clear sticky task queue or report any panics during
        WorkflowTask processing.  Temporal will only append first
        WorkflowTaskFailed event to the history of workflow execution for
        consecutive failures.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/RespondWorkflowTaskFailed",
            request,
            RespondWorkflowTaskFailedResponse,
        )

    async def poll_activity_task_queue(
        self, *, request: PollActivityTaskQueueRequest
    ) -> PollActivityTaskQueueResponse:
        """
        PollActivityTaskQueue is called by application worker to process
        ActivityTask from a specific task queue.  ActivityTask is dispatched to
        callers whenever a ScheduleTask command is made for a workflow
        execution. Application is expected to call
        'RespondActivityTaskCompleted' or 'RespondActivityTaskFailed' once it
        is done processing the task. Application also needs to call
        'RecordActivityTaskHeartbeat' API within 'heartbeatTimeoutSeconds'
        interval to prevent the task from getting timed out.  An event
        'ActivityTaskStarted' event is also written to workflow execution
        history before the ActivityTask is dispatched to application worker.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/PollActivityTaskQueue",
            request,
            PollActivityTaskQueueResponse,
        )

    async def record_activity_task_heartbeat(
        self, *, request: RecordActivityTaskHeartbeatRequest
    ) -> RecordActivityTaskHeartbeatResponse:
        """
        RecordActivityTaskHeartbeat is called by application worker while it is
        processing an ActivityTask.  If worker fails to heartbeat within
        'heartbeatTimeoutSeconds' interval for the ActivityTask, then it will
        be marked as timedout and 'ActivityTaskTimedOut' event will be written
        to the workflow history.  Calling 'RecordActivityTaskHeartbeat' will
        fail with 'NotFoundFailure' in such situations.  Use the 'taskToken'
        provided as response of PollActivityTaskQueue API call for heart
        beating.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/RecordActivityTaskHeartbeat",
            request,
            RecordActivityTaskHeartbeatResponse,
        )

    async def record_activity_task_heartbeat_by_id(
        self, *, request: RecordActivityTaskHeartbeatByIdRequest
    ) -> RecordActivityTaskHeartbeatByIdResponse:
        """
        (-- api-linter: core::0136::prepositions=disabled     aip.dev/not-
        precedent: "By" is used to indicate request type. --)
        RecordActivityTaskHeartbeatById is called by application worker while
        it is processing an ActivityTask.  If worker fails to heartbeat within
        'heartbeatTimeoutSeconds' interval for the ActivityTask, then it will
        be marked as timed out and 'ActivityTaskTimedOut' event will be written
        to the workflow history.  Calling 'RecordActivityTaskHeartbeatById'
        will fail with 'NotFoundFailure' in such situations.  Instead of using
        'taskToken' like in RecordActivityTaskHeartbeat, use Namespace,
        WorkflowId and ActivityId
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/RecordActivityTaskHeartbeatById",
            request,
            RecordActivityTaskHeartbeatByIdResponse,
        )

    async def respond_activity_task_completed(
        self, *, request: RespondActivityTaskCompletedRequest
    ) -> RespondActivityTaskCompletedResponse:
        """
        RespondActivityTaskCompleted is called by application worker when it is
        done processing an ActivityTask.  It will result in a new
        'ActivityTaskCompleted' event being written to the workflow history and
        a new WorkflowTask created for the workflow so new commands could be
        made.  Use the 'taskToken' provided as response of
        PollActivityTaskQueue API call for completion. It fails with
        'NotFoundFailure' if the taskToken is not valid anymore due to activity
        timeout.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/RespondActivityTaskCompleted",
            request,
            RespondActivityTaskCompletedResponse,
        )

    async def respond_activity_task_completed_by_id(
        self, *, request: RespondActivityTaskCompletedByIdRequest
    ) -> RespondActivityTaskCompletedByIdResponse:
        """
        (-- api-linter: core::0136::prepositions=disabled     aip.dev/not-
        precedent: "By" is used to indicate request type. --)
        RespondActivityTaskCompletedById is called by application worker when
        it is done processing an ActivityTask. It will result in a new
        'ActivityTaskCompleted' event being written to the workflow history and
        a new WorkflowTask created for the workflow so new commands could be
        made.  Similar to RespondActivityTaskCompleted but use Namespace,
        WorkflowId and ActivityId instead of 'taskToken' for completion. It
        fails with 'NotFoundFailure' if the these Ids are not valid anymore due
        to activity timeout.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/RespondActivityTaskCompletedById",
            request,
            RespondActivityTaskCompletedByIdResponse,
        )

    async def respond_activity_task_failed(
        self, *, request: RespondActivityTaskFailedRequest
    ) -> RespondActivityTaskFailedResponse:
        """
        RespondActivityTaskFailed is called by application worker when it is
        done processing an ActivityTask.  It will result in a new
        'ActivityTaskFailed' event being written to the workflow history and a
        new WorkflowTask created for the workflow instance so new commands
        could be made.  Use the 'taskToken' provided as response of
        PollActivityTaskQueue API call for completion. It fails with
        'NotFoundFailure' if the taskToken is not valid anymore due to activity
        timeout.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/RespondActivityTaskFailed",
            request,
            RespondActivityTaskFailedResponse,
        )

    async def respond_activity_task_failed_by_id(
        self, *, request: RespondActivityTaskFailedByIdRequest
    ) -> RespondActivityTaskFailedByIdResponse:
        """
        (-- api-linter: core::0136::prepositions=disabled     aip.dev/not-
        precedent: "By" is used to indicate request type. --)
        RespondActivityTaskFailedById is called by application worker when it
        is done processing an ActivityTask. It will result in a new
        'ActivityTaskFailed' event being written to the workflow history and a
        new WorkflowTask created for the workflow instance so new commands
        could be made.  Similar to RespondActivityTaskFailed but use Namespace,
        WorkflowId and ActivityId instead of 'taskToken' for completion. It
        fails with 'NotFoundFailure' if the these Ids are not valid anymore due
        to activity timeout.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/RespondActivityTaskFailedById",
            request,
            RespondActivityTaskFailedByIdResponse,
        )

    async def respond_activity_task_canceled(
        self, *, request: RespondActivityTaskCanceledRequest
    ) -> RespondActivityTaskCanceledResponse:
        """
        RespondActivityTaskCanceled is called by application worker when it is
        successfully canceled an ActivityTask.  It will result in a new
        'ActivityTaskCanceled' event being written to the workflow history and
        a new WorkflowTask created for the workflow instance so new commands
        could be made.  Use the 'taskToken' provided as response of
        PollActivityTaskQueue API call for completion. It fails with
        'NotFoundFailure' if the taskToken is not valid anymore due to activity
        timeout.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/RespondActivityTaskCanceled",
            request,
            RespondActivityTaskCanceledResponse,
        )

    async def respond_activity_task_canceled_by_id(
        self, *, request: RespondActivityTaskCanceledByIdRequest
    ) -> RespondActivityTaskCanceledByIdResponse:
        """
        (-- api-linter: core::0136::prepositions=disabled     aip.dev/not-
        precedent: "By" is used to indicate request type. --)
        RespondActivityTaskCanceledById is called by application worker when it
        is successfully canceled an ActivityTask. It will result in a new
        'ActivityTaskCanceled' event being written to the workflow history and
        a new WorkflowTask created for the workflow instance so new commands
        could be made.  Similar to RespondActivityTaskCanceled but use
        Namespace, WorkflowId and ActivityId instead of 'taskToken' for
        completion. It fails with 'NotFoundFailure' if the these Ids are not
        valid anymore due to activity timeout.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/RespondActivityTaskCanceledById",
            request,
            RespondActivityTaskCanceledByIdResponse,
        )

    async def request_cancel_workflow_execution(
        self, *, request: RequestCancelWorkflowExecutionRequest
    ) -> RequestCancelWorkflowExecutionResponse:
        """
        RequestCancelWorkflowExecution is called by application worker when it
        wants to request cancellation of a workflow instance. It will result in
        a new 'WorkflowExecutionCancelRequested' event being written to the
        workflow history and a new WorkflowTask created for the workflow
        instance so new commands could be made. It fails with 'NotFoundFailure'
        if the workflow is not valid anymore due to completion or doesn't
        exist.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/RequestCancelWorkflowExecution",
            request,
            RequestCancelWorkflowExecutionResponse,
        )

    async def signal_workflow_execution(
        self, *, request: SignalWorkflowExecutionRequest
    ) -> SignalWorkflowExecutionResponse:
        """
        SignalWorkflowExecution is used to send a signal event to running
        workflow execution.  This results in WorkflowExecutionSignaled event
        recorded in the history and a workflow task being created for the
        execution.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/SignalWorkflowExecution",
            request,
            SignalWorkflowExecutionResponse,
        )

    async def signal_with_start_workflow_execution(
        self, *, request: SignalWithStartWorkflowExecutionRequest
    ) -> SignalWithStartWorkflowExecutionResponse:
        """
        (-- api-linter: core::0136::prepositions=disabled     aip.dev/not-
        precedent: "With" is used to indicate combined operation. --)
        SignalWithStartWorkflowExecution is used to ensure sending signal to a
        workflow. If the workflow is running, this results in
        WorkflowExecutionSignaled event being recorded in the history and a
        workflow task being created for the execution. If the workflow is not
        running or not found, this results in WorkflowExecutionStarted and
        WorkflowExecutionSignaled events being recorded in history, and a
        workflow task being created for the execution
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/SignalWithStartWorkflowExecution",
            request,
            SignalWithStartWorkflowExecutionResponse,
        )

    async def reset_workflow_execution(
        self, *, request: ResetWorkflowExecutionRequest
    ) -> ResetWorkflowExecutionResponse:
        """
        ResetWorkflowExecution reset an existing workflow execution to
        WorkflowTaskCompleted event(exclusive). And it will immediately
        terminating the current execution instance.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/ResetWorkflowExecution",
            request,
            ResetWorkflowExecutionResponse,
        )

    async def terminate_workflow_execution(
        self, *, request: TerminateWorkflowExecutionRequest
    ) -> TerminateWorkflowExecutionResponse:
        """
        TerminateWorkflowExecution terminates an existing workflow execution by
        recording WorkflowExecutionTerminated event in the history and
        immediately terminating the execution instance.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/TerminateWorkflowExecution",
            request,
            TerminateWorkflowExecutionResponse,
        )

    async def list_open_workflow_executions(
        self, *, request: ListOpenWorkflowExecutionsRequest
    ) -> ListOpenWorkflowExecutionsResponse:
        """
        ListOpenWorkflowExecutions is a visibility API to list the open
        executions in a specific namespace.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/ListOpenWorkflowExecutions",
            request,
            ListOpenWorkflowExecutionsResponse,
        )

    async def list_closed_workflow_executions(
        self, *, request: ListClosedWorkflowExecutionsRequest
    ) -> ListClosedWorkflowExecutionsResponse:
        """
        ListClosedWorkflowExecutions is a visibility API to list the closed
        executions in a specific namespace.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/ListClosedWorkflowExecutions",
            request,
            ListClosedWorkflowExecutionsResponse,
        )

    async def list_workflow_executions(
        self, *, request: ListWorkflowExecutionsRequest
    ) -> ListWorkflowExecutionsResponse:
        """
        ListWorkflowExecutions is a visibility API to list workflow executions
        in a specific namespace.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/ListWorkflowExecutions",
            request,
            ListWorkflowExecutionsResponse,
        )

    async def list_archived_workflow_executions(
        self, *, request: ListArchivedWorkflowExecutionsRequest
    ) -> ListArchivedWorkflowExecutionsResponse:
        """
        ListArchivedWorkflowExecutions is a visibility API to list archived
        workflow executions in a specific namespace.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/ListArchivedWorkflowExecutions",
            request,
            ListArchivedWorkflowExecutionsResponse,
        )

    async def scan_workflow_executions(
        self, *, request: ScanWorkflowExecutionsRequest
    ) -> ScanWorkflowExecutionsResponse:
        """
        ScanWorkflowExecutions is a visibility API to list large amount of
        workflow executions in a specific namespace without order.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/ScanWorkflowExecutions",
            request,
            ScanWorkflowExecutionsResponse,
        )

    async def count_workflow_executions(
        self, *, request: CountWorkflowExecutionsRequest
    ) -> CountWorkflowExecutionsResponse:
        """
        CountWorkflowExecutions is a visibility API to count of workflow
        executions in a specific namespace.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/CountWorkflowExecutions",
            request,
            CountWorkflowExecutionsResponse,
        )

    async def get_search_attributes(
        self, *, request: GetSearchAttributesRequest
    ) -> GetSearchAttributesResponse:
        """
        GetSearchAttributes is a visibility API to get all legal keys that
        could be used in list APIs
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/GetSearchAttributes",
            request,
            GetSearchAttributesResponse,
        )

    async def respond_query_task_completed(
        self, *, request: RespondQueryTaskCompletedRequest
    ) -> RespondQueryTaskCompletedResponse:
        """
        RespondQueryTaskCompleted is called by application worker to complete a
        QueryTask (which is a WorkflowTask for query) as a result of
        'PollWorkflowTaskQueue' API call. Completing a QueryTask will unblock
        the client call to 'QueryWorkflow' API and return the query result to
        client as a response to 'QueryWorkflow' API call.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/RespondQueryTaskCompleted",
            request,
            RespondQueryTaskCompletedResponse,
        )

    async def reset_sticky_task_queue(
        self, *, request: ResetStickyTaskQueueRequest
    ) -> ResetStickyTaskQueueResponse:
        """
        ResetStickyTaskQueue resets the sticky task queue related information
        in mutable state of a given workflow. Things cleared are: 1.
        StickyTaskQueue 2. StickyScheduleToStartTimeout 3. ClientLibraryVersion
        4. ClientFeatureVersion 5. ClientImpl
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/ResetStickyTaskQueue",
            request,
            ResetStickyTaskQueueResponse,
        )

    async def query_workflow(
        self, *, request: QueryWorkflowRequest
    ) -> QueryWorkflowResponse:
        """
        QueryWorkflow returns query result for a specified workflow execution
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/QueryWorkflow",
            request,
            QueryWorkflowResponse,
        )

    async def describe_workflow_execution(
        self, *, request: DescribeWorkflowExecutionRequest
    ) -> DescribeWorkflowExecutionResponse:
        """
        DescribeWorkflowExecution returns information about the specified
        workflow execution.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/DescribeWorkflowExecution",
            request,
            DescribeWorkflowExecutionResponse,
        )

    async def describe_task_queue(
        self, *, request: DescribeTaskQueueRequest
    ) -> DescribeTaskQueueResponse:
        """
        DescribeTaskQueue returns information about the target task queue,
        right now this API returns the pollers which polled this task queue in
        last few minutes.
        """

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/DescribeTaskQueue",
            request,
            DescribeTaskQueueResponse,
        )

    async def get_cluster_info(
        self, *, request: GetClusterInfoRequest
    ) -> GetClusterInfoResponse:
        """GetClusterInfo returns information about temporal cluster"""

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/GetClusterInfo",
            request,
            GetClusterInfoResponse,
        )

    async def list_task_queue_partitions(
        self, *, request: ListTaskQueuePartitionsRequest
    ) -> ListTaskQueuePartitionsResponse:

        return await self._unary_unary(
            "/temporal.api.workflowservice.v1.WorkflowService/ListTaskQueuePartitions",
            request,
            ListTaskQueuePartitionsResponse,
        )



import json
import re
from typing import List, Optional, Union, Iterable

from temporal.api.common.v1 import Payload, Payloads


def camel_to_snake(name):
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()


def snake_to_camel(snake_str):
    components = snake_str.split('_')
    # We capitalize the first letter of each component except the first one
    # with the 'title' method and join them together.
    return components[0] + ''.join(x.title() for x in components[1:])


def snake_to_title(snake_str):
    components = snake_str.split('_')
    return ''.join(x.title() for x in components)


METADATA_ENCODING_KEY = "encoding"

METADATA_ENCODING_NULL_NAME = "binary/null"
METADATA_ENCODING_NULL = METADATA_ENCODING_NULL_NAME.encode("utf-8")
METADATA_ENCODING_RAW_NAME = "binary/plain"
METADATA_ENCODING_RAW = METADATA_ENCODING_RAW_NAME.encode("utf-8")
METADATA_ENCODING_JSON_NAME = "json/plain"
METADATA_ENCODING_JSON = METADATA_ENCODING_JSON_NAME.encode("utf-8")

# TODO: Implement encode/decode for these:
METADATA_ENCODING_PROTOBUF_JSON_NAME = "json/protobuf"
METADATA_ENCODING_PROTOBUF_JSON = METADATA_ENCODING_PROTOBUF_JSON_NAME.encode("utf-8")
METADATA_ENCODING_PROTOBUF_NAME = "binary/protobuf"
METADATA_ENCODING_PROTOBUF = METADATA_ENCODING_PROTOBUF_NAME.encode('utf-8')


def encode_null(value: object) -> Optional[Payload]:
    if value is None:
        p: Payload = Payload()
        p.metadata = {METADATA_ENCODING_KEY: METADATA_ENCODING_NULL}
        p.data = bytes()
        return p
    else:
        return None


# noinspection PyUnusedLocal
def decode_null(payload: Payload) -> object:
    return None


def encode_binary(value: object) -> Optional[Payload]:
    if isinstance(value, bytes):
        p: Payload = Payload()
        p.metadata = {METADATA_ENCODING_KEY: METADATA_ENCODING_RAW}
        p.data = value
        return p
    else:
        return None


def decode_binary(payload: Payload) -> object:
    return payload.data


def encode_json_string(value: object) -> Payload:
    # TODO:
    # mapper.configure(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS, false);
    # mapper.registerModule(new JavaTimeModule());
    # mapper.setVisibility(PropertyAccessor.FIELD, JsonAutoDetect.Visibility.ANY);
    p: Payload = Payload()
    p.metadata = {METADATA_ENCODING_KEY: METADATA_ENCODING_JSON}
    p.data = json.dumps(value).encode("utf-8")
    return p


def decode_json_string(payload: Payload) -> object:
    # TODO:
    # mapper.configure(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS, false);
    # mapper.registerModule(new JavaTimeModule());
    # mapper.setVisibility(PropertyAccessor.FIELD, JsonAutoDetect.Visibility.ANY);
    b = str(payload.data, "utf-8")
    return json.loads(b)


ENCODINGS = [
    encode_null,
    encode_binary,
    encode_json_string
]


DECODINGS = {
    METADATA_ENCODING_NULL: decode_null,
    METADATA_ENCODING_RAW: decode_binary,
    METADATA_ENCODING_JSON: decode_json_string
}




# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: temporal/api/common/v1/message.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import timedelta
from typing import Dict, List

import betterproto

from temporal.api.enums import v1 as v1enums


@dataclass
class DataBlob(betterproto.Message):
    encoding_type: v1enums.EncodingType = betterproto.enum_field(1)
    data: bytes = betterproto.bytes_field(2)


@dataclass
class Payloads(betterproto.Message):
    payloads: List["Payload"] = betterproto.message_field(1)


@dataclass
class Payload(betterproto.Message):
    metadata: Dict[str, bytes] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_BYTES
    )
    data: bytes = betterproto.bytes_field(2)


@dataclass
class SearchAttributes(betterproto.Message):
    indexed_fields: Dict[str, "Payload"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )


@dataclass
class Memo(betterproto.Message):
    fields: Dict[str, "Payload"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )


@dataclass
class Header(betterproto.Message):
    fields: Dict[str, "Payload"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )


@dataclass
class WorkflowExecution(betterproto.Message):
    workflow_id: str = betterproto.string_field(1)
    run_id: str = betterproto.string_field(2)


@dataclass
class WorkflowType(betterproto.Message):
    name: str = betterproto.string_field(1)


@dataclass
class ActivityType(betterproto.Message):
    name: str = betterproto.string_field(1)


@dataclass
class RetryPolicy(betterproto.Message):
    # Interval of the first retry. If retryBackoffCoefficient is 1.0 then it is
    # used for all retries.
    initial_interval: timedelta = betterproto.message_field(1)
    # Coefficient used to calculate the next retry interval. The next retry
    # interval is previous interval multiplied by the coefficient. Must be 1 or
    # larger.
    backoff_coefficient: float = betterproto.double_field(2)
    # Maximum interval between retries. Exponential backoff leads to interval
    # increase. This value is the cap of the increase. Default is 100x of the
    # initial interval.
    maximum_interval: timedelta = betterproto.message_field(3)
    # Maximum number of attempts. When exceeded the retries stop even if not
    # expired yet. 1 disables retries. 0 means unlimited (up to the timeouts)
    maximum_attempts: int = betterproto.int32_field(4)
    # Non-Retryable errors types. Will stop retrying if error type matches this
    # list.
    non_retryable_error_types: List[str] = betterproto.string_field(5)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: temporal/api/namespace/v1/message.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime, timedelta
from typing import Dict

import betterproto

from temporal.api.enums import v1 as v1enums


@dataclass
class NamespaceInfo(betterproto.Message):
    name: str = betterproto.string_field(1)
    state: v1enums.NamespaceState = betterproto.enum_field(2)
    description: str = betterproto.string_field(3)
    owner_email: str = betterproto.string_field(4)
    # A key-value map for any customized purpose.
    data: Dict[str, str] = betterproto.map_field(
        5, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )
    id: str = betterproto.string_field(6)


@dataclass
class NamespaceConfig(betterproto.Message):
    workflow_execution_retention_ttl: timedelta = betterproto.message_field(1)
    bad_binaries: "BadBinaries" = betterproto.message_field(2)
    # If unspecified (ARCHIVAL_STATE_UNSPECIFIED) then default server
    # configuration is used.
    history_archival_state: v1enums.ArchivalState = betterproto.enum_field(3)
    history_archival_uri: str = betterproto.string_field(4)
    # If unspecified (ARCHIVAL_STATE_UNSPECIFIED) then default server
    # configuration is used.
    visibility_archival_state: v1enums.ArchivalState = betterproto.enum_field(5)
    visibility_archival_uri: str = betterproto.string_field(6)


@dataclass
class BadBinaries(betterproto.Message):
    binaries: Dict[str, "BadBinaryInfo"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )


@dataclass
class BadBinaryInfo(betterproto.Message):
    reason: str = betterproto.string_field(1)
    operator: str = betterproto.string_field(2)
    create_time: datetime = betterproto.message_field(3)


@dataclass
class UpdateNamespaceInfo(betterproto.Message):
    description: str = betterproto.string_field(1)
    owner_email: str = betterproto.string_field(2)
    # A key-value map for any customized purpose.
    data: Dict[str, str] = betterproto.map_field(
        3, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )



import json
import logging
import traceback
from typing import Optional

import tblib  # type: ignore

from temporal.api.failure.v1 import Failure, ApplicationFailureInfo

THIS_SOURCE = "PythonSDK"

logger = logging.getLogger(__name__)


class ExternalException(Exception):
    def __init__(self, details):
        super().__init__(details)

    @property
    def details(self):
        return self.args[0]


def exception_class_fqn(o):
    # Copied from: https://stackoverflow.com/a/2020083
    module = o.__class__.__module__
    # if module is None or module == str.__class__.__module__:
    #     return o.__class__.__name__  # Avoid reporting __builtin__
    # else:
    return module + '.' + o.__class__.__name__


def import_class_from_string(path):
    # Taken from: https://stackoverflow.com/a/30042585
    from importlib import import_module
    module_path, _, class_name = path.rpartition('.')
    mod = import_module(module_path)
    klass = getattr(mod, class_name)
    return klass


def serialize_exception(ex: BaseException) -> Failure:
    failure = Failure()
    failure.message = str(ex)
    failure.source = THIS_SOURCE
    # TODO: support chaining?
    # failure.cause = ???
    exception_cls_name: str = exception_class_fqn(ex)
    failure.application_failure_info = ApplicationFailureInfo()
    failure.application_failure_info.type = exception_class_fqn(ex)
    tb = "".join(traceback.format_exception(type(ex), ex, ex.__traceback__))
    failure.stack_trace = json.dumps({
        "class": exception_cls_name,
        "args": ex.args,
        "traceback": tb,
    })
    return failure


"""
TODO: Need unit testing for this
"""
def deserialize_exception(details: Failure) -> Exception:
    """
    TODO: Support built-in types like Exception
    """
    exception: Optional[Exception] = None
    source = details.source
    exception_cls_name: str = details.application_failure_info.type

    if source == THIS_SOURCE and exception_cls_name:
        details_dict = json.loads(details.stack_trace)
        try:
            klass = import_class_from_string(exception_cls_name)
            exception = klass(*details_dict["args"])
            traceback_string: str = details_dict["traceback"]
            # when complete_exceptionally() is invoked, the Exception has no
            # traceback, so don't try to deserialize the traceback if there is none
            if len(traceback_string.split("\n")) > 2:
                t = tblib.Traceback.from_string(traceback_string)
                assert exception is not None
                exception.with_traceback(t.as_traceback())
        except Exception as e:
            exception = None
            logger.error("Failed to deserialize exception (details=%s) cause=%r", details_dict, e)

    if not exception:
        # TODO: Better to deserialize details
        return ExternalException(details)
    else:
        return exception


def failure_to_str(f: Failure) -> str:
    return f.to_json()


def str_to_failure(s: str) -> Failure:
    f = Failure()
    return f.from_json(s)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: temporal/api/failure/v1/message.proto
# plugin: python-betterproto
from dataclasses import dataclass

import betterproto

from temporal.api.common import v1 as v1common
from temporal.api.enums import v1 as v1enums


@dataclass
class ApplicationFailureInfo(betterproto.Message):
    type: str = betterproto.string_field(1)
    non_retryable: bool = betterproto.bool_field(2)
    details: v1common.Payloads = betterproto.message_field(3)


@dataclass
class TimeoutFailureInfo(betterproto.Message):
    timeout_type: v1enums.TimeoutType = betterproto.enum_field(1)
    last_heartbeat_details: v1common.Payloads = betterproto.message_field(2)


@dataclass
class CanceledFailureInfo(betterproto.Message):
    details: v1common.Payloads = betterproto.message_field(1)


@dataclass
class TerminatedFailureInfo(betterproto.Message):
    pass


@dataclass
class ServerFailureInfo(betterproto.Message):
    non_retryable: bool = betterproto.bool_field(1)


@dataclass
class ResetWorkflowFailureInfo(betterproto.Message):
    last_heartbeat_details: v1common.Payloads = betterproto.message_field(1)


@dataclass
class ActivityFailureInfo(betterproto.Message):
    scheduled_event_id: int = betterproto.int64_field(1)
    started_event_id: int = betterproto.int64_field(2)
    identity: str = betterproto.string_field(3)
    activity_type: v1common.ActivityType = betterproto.message_field(4)
    activity_id: str = betterproto.string_field(5)
    retry_state: v1enums.RetryState = betterproto.enum_field(6)


@dataclass
class ChildWorkflowExecutionFailureInfo(betterproto.Message):
    namespace: str = betterproto.string_field(1)
    workflow_execution: v1common.WorkflowExecution = betterproto.message_field(2)
    workflow_type: v1common.WorkflowType = betterproto.message_field(3)
    initiated_event_id: int = betterproto.int64_field(4)
    started_event_id: int = betterproto.int64_field(5)
    retry_state: v1enums.RetryState = betterproto.enum_field(6)


@dataclass
class Failure(betterproto.Message):
    message: str = betterproto.string_field(1)
    source: str = betterproto.string_field(2)
    stack_trace: str = betterproto.string_field(3)
    cause: "Failure" = betterproto.message_field(4)
    application_failure_info: "ApplicationFailureInfo" = betterproto.message_field(
        5, group="failure_info"
    )
    timeout_failure_info: "TimeoutFailureInfo" = betterproto.message_field(
        6, group="failure_info"
    )
    canceled_failure_info: "CanceledFailureInfo" = betterproto.message_field(
        7, group="failure_info"
    )
    terminated_failure_info: "TerminatedFailureInfo" = betterproto.message_field(
        8, group="failure_info"
    )
    server_failure_info: "ServerFailureInfo" = betterproto.message_field(
        9, group="failure_info"
    )
    reset_workflow_failure_info: "ResetWorkflowFailureInfo" = betterproto.message_field(
        10, group="failure_info"
    )
    activity_failure_info: "ActivityFailureInfo" = betterproto.message_field(
        11, group="failure_info"
    )
    child_workflow_execution_failure_info: "ChildWorkflowExecutionFailureInfo" = betterproto.message_field(
        12, group="failure_info"
    )



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: temporal/api/filter/v1/message.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime

import betterproto

from temporal.api.enums import v1 as v1enums


@dataclass
class WorkflowExecutionFilter(betterproto.Message):
    workflow_id: str = betterproto.string_field(1)
    run_id: str = betterproto.string_field(2)


@dataclass
class WorkflowTypeFilter(betterproto.Message):
    name: str = betterproto.string_field(1)


@dataclass
class StartTimeFilter(betterproto.Message):
    earliest_time: datetime = betterproto.message_field(1)
    latest_time: datetime = betterproto.message_field(2)


@dataclass
class StatusFilter(betterproto.Message):
    status: v1enums.WorkflowExecutionStatus = betterproto.enum_field(1)



import inspect
from dataclasses import dataclass, field
from datetime import timedelta
from typing import Callable, List

from temporal.api.common.v1 import RetryPolicy, ActivityType, Payloads



def get_activity_method_name(method: Callable):
    return "::".join(method.__qualname__.split(".")[-2:])


@dataclass
class RetryParameters:
    initial_interval: timedelta = None
    backoff_coefficient: float = None
    maximum_interval: timedelta = None
    maximum_attempts: int = None
    non_retryable_error_types: List[str] = field(default_factory=list)

    def to_retry_policy(self) -> RetryPolicy:
        policy = RetryPolicy()
        policy.initial_interval = self.initial_interval
        policy.backoff_coefficient = self.backoff_coefficient
        policy.maximum_interval = self.maximum_interval
        policy.maximum_attempts = self.maximum_attempts
        policy.non_retryable_error_types = self.non_retryable_error_types
        return policy


@dataclass
class ExecuteActivityParameters:
    fn: Callable = None
    activity_id: str = ""
    activity_type: ActivityType = None
    heartbeat_timeout: timedelta = None
    input: Payloads = None
    schedule_to_close_timeout: timedelta = None
    schedule_to_start_timeout: timedelta = None
    start_to_close_timeout: timedelta = None
    task_queue: str = ""
    retry_parameters: RetryParameters = None


def activity_method(func: Callable = None, name: str = "", schedule_to_close_timeout: timedelta = None,
                    schedule_to_start_timeout: timedelta = None, start_to_close_timeout: timedelta = None,
                    heartbeat_timeout: timedelta = None, task_queue: str = "", retry_parameters: RetryParameters = None):
    def wrapper(fn: Callable):
        # noinspection PyProtectedMember
        async def stub_activity_fn(self, *args):
            from .async_activity import Async
            from .decision_loop import ActivityFuture
            future: ActivityFuture = Async.function_with_self(stub_activity_fn, self, *args)
            return await future.wait_for_result()

        if not task_queue:
            raise Exception("task_queue parameter is mandatory")

        execute_parameters = ExecuteActivityParameters()
        execute_parameters.fn = fn
        execute_parameters.activity_type = ActivityType()
        execute_parameters.activity_type.name = name if name else get_activity_method_name(fn)
        execute_parameters.schedule_to_close_timeout = schedule_to_close_timeout
        execute_parameters.schedule_to_start_timeout = schedule_to_start_timeout
        execute_parameters.start_to_close_timeout = start_to_close_timeout
        execute_parameters.heartbeat_timeout = heartbeat_timeout
        execute_parameters.task_queue = task_queue
        execute_parameters.retry_parameters = retry_parameters
        # noinspection PyTypeHints
        stub_activity_fn._execute_parameters = execute_parameters  # type: ignore
        return stub_activity_fn

    if func and inspect.isfunction(func):
        raise Exception("activity_method must be called with arguments")
    else:
        return wrapper


@dataclass
class ActivityOptions:
    schedule_to_close_timeout: timedelta = None
    schedule_to_start_timeout: timedelta = None
    start_to_close_timeout: timedelta = None
    heartbeat_timeout: timedelta = None
    task_queue: str = None

    def fill_execute_activity_parameters(self, execute_parameters: ExecuteActivityParameters):
        if self.schedule_to_close_timeout is not None:
            execute_parameters.schedule_to_close_timeout = self.schedule_to_close_timeout
        if self.schedule_to_start_timeout is not None:
            execute_parameters.schedule_to_start_timeout = self.schedule_to_start_timeout
        if self.start_to_close_timeout is not None:
            execute_parameters.start_to_close_timeout = self.start_to_close_timeout
        if self.heartbeat_timeout is not None:
            execute_parameters.heartbeat_timeout = self.heartbeat_timeout
        if self.task_queue is not None:
            execute_parameters.task_queue = self.task_queue


@dataclass
class UntypedActivityStub:
    _decision_context: object = None
    _retry_parameters: RetryParameters = None
    _activity_options: ActivityOptions = None

    async def execute(self, activity_name: str, *args):
        f = await self.execute_async(activity_name, *args)
        return await f.wait_for_result()

    async def execute_async(self, activity_name: str, *args):
        from .async_activity import Async
        execute_parameters = ExecuteActivityParameters()
        execute_parameters.activity_type = ActivityType()
        execute_parameters.activity_type.name = activity_name
        return Async.call(self, execute_parameters, args)



import setuptools
from pkg_resources import parse_requirements

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="temporal-python-sdk",
    version="1.0.16",
    author="Mohammed Firdaus",
    author_email="firdaus.halim@gmail.com",
    description="Unofficial Python SDK for the Temporal Workflow Engine",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/firdaus/temporal-python-sdk",
    packages=setuptools.find_packages(exclude=["cadence", "tests", "cadence.*", "tests.*"]),
    install_requires=[
        "betterproto-for-temporal-python-sdk==1.2.5",
        "dataclasses-json==0.3.8",
        "grpcio==1.30.0",
        "grpclib==0.3.2",
        "h2==3.2.0",
        "more-itertools==7.0.0",
        "pytz==2020.1",
        "tblib==1.6.0"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    include_package_data=True
)
